@@ -23,9 +23,11 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*                                                                           */
 /*****************************************************************************/
+#include <string.h>
+#include <stdlib.h>
 #include <stdio.h>
 
-#define C2LOUT_VERSION	"c2lout Version 1.0 (March 1994)"
+#define C2LOUT_VERSION	"c2lout Version 3.15 (May 1999)"
 #define	BOOLEAN		unsigned
 #define	FALSE		0
 #define	TRUE		1
@@ -64,11 +66,11 @@ extern void ProcessStandAlone(char *fname, FILE *in_fp, FILE *out_fp);
     fprintf(err_fp, "c2lout %s %d,%d: ", file_name, line_num, line_pos);\
   else									\
     fprintf(err_fp, "c2lout: ");					\
-  fprintf(err_fp, str);						\
+  fprintf(err_fp, str);							\
   fprintf(err_fp, "\n");						\
 }
 
-#define Error1(str, arg)							\
+#define Error1(str, arg)						\
 {									\
   if( line_num > 0 )							\
     fprintf(err_fp, "c2lout %s %d,%d: ", file_name, line_num, line_pos);\
@@ -100,7 +102,7 @@ extern void ProcessStandAlone(char *fname, FILE *in_fp, FILE *out_fp);
 /*                                                                           */
 /*****************************************************************************/
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 { FILE *in_fp, *out_fp = stdout;
   BOOLEAN at_least_one_file, raw_seen;  int i;
   char *infilename, *outfilename, *errfilename, *str;
@@ -136,7 +138,7 @@ void main(int argc, char *argv[])
      
 	/* read name of input file */
 	if( !raw_seen )
-	{ Error0("-i illegal with -r");
+	{ Error0("-i illegal without -r");
 	  exit(1);
 	}
 	if( in_fp != NULL )
@@ -372,7 +374,7 @@ void main(int argc, char *argv[])
   else if( at_least_one_file )
     fprintf(out_fp == NULL ? stdout : out_fp, "@End @Text\n");
 
-  exit(0);
+  return 0;
 } /* end main */
 
 
@@ -442,7 +444,8 @@ void ProcessStandAlone(char *fname, FILE *in_fp, FILE *out_fp)
 		 
 
     /* now print the initial @Use clauses etc.*/
-    fprintf(out_fp, "@SysInclude { cdoc }\n");
+    fprintf(out_fp, "@SysInclude { cprint }\n");
+    fprintf(out_fp, "@SysInclude { doc }\n");
     fprintf(out_fp, "@Use { @CP\n");
     fprintf(out_fp, "    style  { %s }\n", style_str);
     fprintf(out_fp, "    font   { %s }\n", font_str);
