@@ -1,7 +1,7 @@
 /*@z08.c:Object Manifest:ReplaceWithSplit()@**********************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -10,7 +10,7 @@
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,21 +20,23 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z08.c                                                      */
 /*  MODULE:       Object Manifest                                            */
 /*  EXTERNS:      Manifest()                                                 */
 /*                                                                           */
 /*****************************************************************************/
-#include "externs"
+#include "externs.h"
+#define line_breaker(g)							\
+  (vspace(g) > 0 || (units(gap(g)) == FRAME_UNIT && width(gap(g)) > FR))
 
 
 /*****************************************************************************/
 /*                                                                           */
 /*  static SetUnderline(x)                                                   */
 /*                                                                           */
-/*  Set underline() flags in object x to TRUE as appropriate.                */
+/*  Set underline() flags in object x to UNDER_ON as appropriate.            */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -48,8 +50,8 @@ static void SetUnderline(OBJECT x)
     }
   }
   debug3(DOM, DDD, "  SetUnderline underline() := %s for %s %s",
-    bool(TRUE), Image(type(x)), EchoObject(x));
-  underline(x) = TRUE;
+    "UNDER_ON", Image(type(x)), EchoObject(x));
+  underline(x) = UNDER_ON;
 } /* end SetUnderline */
 
 
@@ -245,8 +247,10 @@ static OBJECT ManifestCat(OBJECT x, OBJECT env, STYLE *style, OBJECT bthr[2],
 OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand,
 OBJECT *enclose, BOOLEAN fcr)
 { OBJECT bt[2], ft[2], y, link, gaplink, g, first_bt, last_ft, z;
-  int par, perp;  unsigned res_inc;  BOOLEAN still_backing;
+  int par, perp;
+  unsigned res_inc;  BOOLEAN still_backing;
   STYLE new_style;
+  debug1(DOM, DD, "[ ManifestCat(%s)", EchoObject(x));
     
   StyleCopy(new_style, *style);
   if( type(x) == HCAT )
@@ -271,10 +275,8 @@ OBJECT *enclose, BOOLEAN fcr)
   if( bthr[par] )  { New(first_bt, THREAD); }
   else first_bt = nilobj;
   bt[par] = first_bt;
-  /* *** first_bt = bt[par] = bthr[par] ? New(THREAD) : nilobj; *** */
   if( join(gap(g)) )  { New(ft[par], THREAD); }
   else ft[par] = nilobj;
-  /* *** ft[par] = join(gap(g)) ? New(THREAD) : nilobj;         *** */
   still_backing = first_bt != nilobj;
 
   /* manifest y and insinuate any cross-references */
@@ -312,12 +314,6 @@ OBJECT *enclose, BOOLEAN fcr)
 
     /* set bt and ft threads for y */
     last_ft = ft[par];
-
-    /* ***
-    bt[par] = ft[par] ? New(THREAD) : nilobj;
-    ft[par] = g != nilobj ? join(gap(g)) ? New(THREAD) : nilobj
-			  : fthr[par]    ? New(THREAD) : nilobj;
-    *** */
     if( ft[par] ) { New(bt[par], THREAD); }  else bt[par] = nilobj;
     if( g != nilobj )
     { if( join(gap(g)) )  { New(ft[par], THREAD); }  else ft[par] = nilobj;
@@ -337,7 +333,8 @@ OBJECT *enclose, BOOLEAN fcr)
     }
 
     if( bt[par] )	/* then thread lists last_ft and bt[par] must merge */
-    { OBJECT llink, rlink, lthread, rthread;  BOOLEAN goes_through;
+    { OBJECT llink, rlink, lthread, rthread;
+      BOOLEAN goes_through;
       assert( Down(bt[par]) != bt[par], "Manifest: bt[par] no children!" );
       assert( last_ft!=nilobj && Down(last_ft)!=last_ft, "Manifest:last_ft!" );
 
@@ -385,6 +382,7 @@ OBJECT *enclose, BOOLEAN fcr)
   /* export par threads */
   if( fthr[par] )  MergeNode(fthr[par], ft[par]);
   if( bthr[par] )  MergeNode(bthr[par], first_bt);
+  debug0(DOM, DD, "] ManifestCat returning");
   return x;
 } /* end ManifestCat */
 
@@ -482,6 +480,7 @@ OBJECT *enclose, BOOLEAN fcr)
     { Error(8, 11, "replacing unknown %s option %s by %s",
 	WARN, &fpos(tag), KW_CASE, string(tag), string(firsttag));
       res = firstres;
+      debug1(DGP, D, "  res = %s", EchoObject(res));
     }
     else
     { Error(8, 12, "%s deleted (choice %s unknown)",
@@ -612,6 +611,8 @@ OBJECT *enclose, BOOLEAN fcr)
 
   sym = actual(x);
   StyleCopy(save_style(x), *style);
+  debugcond2(DOM, D, StringEqual(SymName(sym), "@Section"),
+     "manifesting %s at %s", SymName(sym), EchoFilePos(&fpos(x)));
   debug1(DOM, DD,  "  [ manifesting closure %s", SymName(sym));
 
   /* enclose, if required */
@@ -624,7 +625,7 @@ OBJECT *enclose, BOOLEAN fcr)
     Link(par, x);
     x = *enclose;
     *enclose = nilobj;
-    debug1(DHY, D, "  Manifest/enclose: %s", EchoObject(x));
+    debug1(DHY, DD, "  Manifest/enclose: %s", EchoObject(x));
     x = Manifest(x, env, style, bthr, fthr, target, crs, ok, FALSE, enclose, fcr);
     debug0(DOM, DD,  "  ] returning from manifesting closure (enclose)");
     return x;
@@ -644,6 +645,12 @@ OBJECT *enclose, BOOLEAN fcr)
       if( is_tag(actual(y)) || is_key(actual(y)) )
       {	z = Manifest(z, env, style, nbt, nft, &ntarget, crs, FALSE, FALSE, &nenclose, fcr);
 	z = ReplaceWithTidy(z, TRUE);
+	if( !is_word(type(z)) )
+	{
+	  debug2(ANY, D, "z = %s %s", Image(type(z)), EchoObject(z));
+	  Error(8, 41, "this %s is not a sequence of one or more words", FATAL,
+	    &fpos(y), SymName(actual(y)));
+	}
       }
       else if( type(z) == NEXT )
       {	z = Manifest(z, env, style, nbt, nft, &ntarget, crs, FALSE, FALSE, &nenclose, fcr);
@@ -726,6 +733,8 @@ OBJECT *enclose, BOOLEAN fcr)
   }
   else hold_env2 = nilobj;
 
+  debug3(DOM, DD, " expansion: has_target %s, indefinite %s, recursive %s",
+    bool(has_target(sym)), bool(indefinite(sym)), bool(recursive(sym)));
   if( has_target(sym) && !need_expand )
   {
     /* convert symbols with targets to unsized galleys */
@@ -852,15 +861,16 @@ OBJECT *enclose, BOOLEAN fcr)
 /*  the result; so expansion will stop at a CROSS or FORCE_CROSS object.     */
 /*                                                                           */
 /*  A postcondition of Manifest() is that the underline() flag is set to     */
-/*  either TRUE or FALSE in every WORD, every QWORD, and every child of      */
-/*  every ACAT, including the gaps.  This can be verified by checking that   */
-/*  the WORD and QWORD cases set underline() to FALSE, and the ACAT case     */
-/*  sets every child of the ACAT to FALSE.  To see that the correct subset   */
-/*  of these flags gets changed to TRUE, consult SetUnderline().  The        */
-/*  underline() flag is undefined otherwise.                                 */
+/*  either UNDER_ON or UNDER_OFF in every WORD, every QWORD, and every child */
+/*  of every ACAT, including the gaps.  This can be verified by checking     */
+/*  that the WORD and QWORD cases set underline() to UNDER_OFF, and the ACAT */
+/*  case sets every child of the ACAT to UNDER_OFF.  To see that the correct */
+/*  subset of these flags gets changed to UNDER_ON, consult SetUnderline().  */
+/*  The underline() flag is undefined otherwise, and should have value       */
+/*  UNDER_UNDEF.                                                             */
 /*                                                                           */
 /*****************************************************************************/
-#define MAX_DEPTH 300
+#define MAX_DEPTH 500
 
 OBJECT Manifest(OBJECT x, OBJECT env, STYLE *style, OBJECT bthr[2],
 OBJECT fthr[2], OBJECT *target, OBJECT *crs, BOOLEAN ok, BOOLEAN need_expand,
@@ -870,11 +880,11 @@ OBJECT *enclose, BOOLEAN fcr)
   OBJECT link1, link2, x1, x2, y1, y2;
   int par, num1, num2;  GAP res_gap;  unsigned res_inc;  STYLE new_style;
   BOOLEAN done, multiline;  FULL_CHAR ch;  float scale_factor;
+  static int depth = 0;
 #if DEBUG_ON
   static unsigned int debug_type[MAX_DEPTH];
   static OBJECT	      debug_actual[MAX_DEPTH];
   static int	      debug_lnum[MAX_DEPTH];
-  static int depth = 0;
   BOOLEAN eee = (*enclose != nilobj);
   debug_type[depth] = type(x);
   debug_lnum[depth] = line_num(fpos(x));
@@ -892,6 +902,13 @@ OBJECT *enclose, BOOLEAN fcr)
     }
     Error(8, 23, "exiting now", FATAL, &fpos(x));
   }
+#else
+  depth++;
+  if( depth == MAX_DEPTH )
+  {
+    Error(8, 40, "maximum depth of symbol expansion (%d) reached",
+      FATAL, &fpos(x), MAX_DEPTH);
+  }
 #endif
 
   debug2(DOM, DD,   "[Manifest(%s %s )", Image(type(x)), EchoObject(x));
@@ -900,7 +917,7 @@ OBJECT *enclose, BOOLEAN fcr)
 	EchoStyle(style), SymName(*target),
 	bthr[COLM] ? " up"    : "",  fthr[COLM] ? " down"  : "",
 	bthr[ROWM] ? " left"  : "",  fthr[ROWM] ? " right" : "");
-  debugcond2(DHY, D, eee, "[ Manifest(%s, *enclose = %s)",
+  debugcond2(DHY, DD, eee, "[ Manifest(%s, *enclose = %s)",
     EchoObject(x), EchoObject(*enclose));
 
   switch( type(x) )
@@ -987,9 +1004,9 @@ OBJECT *enclose, BOOLEAN fcr)
 	word_language(x) = language(*style);
 	word_hyph(x) = hyph_style(*style) == HYPH_ON;
 	debug3(DOM, DDD, "  manfifest/WORD underline() := %s for %s %s",
-	  bool(FALSE), Image(type(x)), EchoObject(x));
-	underline(x) = FALSE;
+	  "UNDER_OFF", Image(type(x)), EchoObject(x));
 	if( small_caps(*style) && ok )  x = MapSmallCaps(x, style);
+	underline(x) = UNDER_OFF;
 	ReplaceWithSplit(x, bthr, fthr);
 	break;
       }
@@ -1021,8 +1038,8 @@ OBJECT *enclose, BOOLEAN fcr)
       }
       else y = Manifest(y, env, &new_style, nbt, nft, target, crs, ok, FALSE, enclose, fcr);
       debug3(DOM, DDD, "  manfifest/ACAT1 underline() := %s for %s %s",
-	bool(FALSE), Image(type(y)), EchoObject(y));
-      underline(y) = FALSE;
+	"UNDER_OFF", Image(type(y)), EchoObject(y));
+      underline(y) = UNDER_OFF;
       /* ??? if( is_word(type(y)) ) */
       if( ok && *crs != nilobj )
       {	
@@ -1039,8 +1056,8 @@ OBJECT *enclose, BOOLEAN fcr)
 	Child(g, gaplink);
 	assert( type(g) == GAP_OBJ, "Manifest ACAT: no GAP_OBJ!" );
         debug3(DOM, DDD, "  manfifest/ACAT2 underline() := %s for %s %s",
-	  bool(FALSE), Image(type(g)), EchoObject(g));
-        underline(g) = FALSE;
+	  "UNDER_OFF", Image(type(g)), EchoObject(g));
+        underline(g) = UNDER_OFF;
 	link = NextDown(gaplink);
 	assert( link != x, "Manifest ACAT: GAP_OBJ is last!" );
 	Child(y, link);
@@ -1057,8 +1074,8 @@ OBJECT *enclose, BOOLEAN fcr)
 	}
 	else y = Manifest(y, env, &new_style, nbt, nft, target, crs, ok, FALSE, enclose, fcr);
         debug3(DOM, DDD, "  manifest/ACAT3 underline() := %s for %s %s",
-	  bool(FALSE), Image(type(y)), EchoObject(y));
-        underline(y) = FALSE;
+	  "UNDER_OFF", Image(type(y)), EchoObject(y));
+        underline(y) = UNDER_OFF;
 
 	/* manifest the gap object */
 	if( Down(g) != g )
@@ -1105,7 +1122,7 @@ OBJECT *enclose, BOOLEAN fcr)
 
 	      /* Lout spacing plus one extra space for sentence end at eoln */
 	      width(gap(g)) = width(gap(g)) * (vspace(g) + hspace(g));
-	      debugcond2(DLS, D, vspace(g) > 0, "  prev = %s %s",
+	      debugcond2(DLS, DD, vspace(g) > 0, "  prev = %s %s",
 		Image(type(prev)), EchoObject(prev));
 	      if( vspace(g) > 0 )
 	      { 
@@ -1121,7 +1138,7 @@ OBJECT *enclose, BOOLEAN fcr)
 		if( is_word(type(z)) )
 		{
 		  for( p = string(z);  *p != '\0';  p++ );
-		  debug4(DLS, D, "  prev = %s, last = %c, LSE = %s, LWES = %s",
+		  debug4(DLS, DD, "  prev = %s, last = %c, LSE = %s, LWES = %s",
 		    EchoObject(z), *(p-1), bool(LanguageSentenceEnds[*(p-1)]),
 		    bool(LanguageWordEndsSentence(z, FALSE)));
 		  if( p != string(z) && LanguageSentenceEnds[*(p-1)]
@@ -1153,7 +1170,7 @@ OBJECT *enclose, BOOLEAN fcr)
 	        if( is_word(type(z)) )
 	        {
 		  for( p = string(z);  *p != '\0';  p++ );
-		  debug4(DLS, D, "  prev = %s, last = %c, LSE = %s, LWES = %s",
+		  debug4(DLS, DD, "  prev = %s, last = %c, LSE = %s, LWES = %s",
 		      EchoObject(z), *(p-1), bool(LanguageSentenceEnds[*(p-1)]),
 		      bool(LanguageWordEndsSentence(z, TRUE)));
 		  if( p != string(z) && LanguageSentenceEnds[*(p-1)]
@@ -1170,7 +1187,7 @@ OBJECT *enclose, BOOLEAN fcr)
 	      break;
 	  }
 	  nobreak(gap(g)) = (width(gap(g)) == 0);
-	  if( vspace(g) > 0 && is_definite(type(y)) )  multiline = TRUE;
+	  if( line_breaker(g) && is_definite(type(y)) )  multiline = TRUE;
 	}
         debug1(DOM, DD, "  in ACAT, gap = %s", EchoLength(width(gap(g))));
 
@@ -1184,6 +1201,8 @@ OBJECT *enclose, BOOLEAN fcr)
 	    word_language(prev) == word_language(y) )
 	    /* no need to compare underline() since both are false */
 	{ unsigned typ;
+	  assert( underline(prev) == UNDER_OFF, "Manifest/ACAT: underline(prev)!" );
+	  assert( underline(y) == UNDER_OFF, "Manifest/ACAT: underline(y)!" );
 	  if( StringLength(string(prev))+StringLength(string(y)) >= MAX_BUFF )
 	    Error(8, 24, "word %s%s is too long",
 	      FATAL, &fpos(prev), string(prev), string(y));
@@ -1194,11 +1213,9 @@ OBJECT *enclose, BOOLEAN fcr)
 	  word_colour(y) = word_colour(prev);
 	  word_language(y) = word_language(prev);
 	  word_hyph(y) = word_hyph(prev);
-	  /* ***
-	  underline(y) = underline(prev);
+	  underline(y) = UNDER_OFF;
           debug3(DOM, DDD, "  manifest/ACAT4 underline() := %s for %s %s",
-	    bool(underline(y)), Image(type(y)), EchoObject(y));
-	  *** */
+	    "UNDER_OFF", Image(type(y)), EchoObject(y));
 	  MoveLink(link, y, CHILD);
 	  DisposeObject(z);
 	  DisposeChild(Up(prev));
@@ -1244,13 +1261,13 @@ OBJECT *enclose, BOOLEAN fcr)
 	  NextDefiniteWithGap(prev_acat, link, y, g, jn);
 	  while( link != prev_acat )
 	  {
-	    if( mode(gap(g)) != NO_MODE && vspace(g) > 0 )
+	    if( mode(gap(g)) != NO_MODE && line_breaker(g) )
 	    { OBJECT glink = PrevDown(Up(g));
 	      debug2(DOM, DD, "lines gap just before definite %s at %s",
 		Image(type(y)), EchoFilePos(&fpos(y)));
 	      MoveLink(NextDown(glink), x, PARENT);
 	      GapCopy(gap(g), line_gap(*style));
-	      width(gap(g)) *= vspace(g);
+	      width(gap(g)) *= find_max(1, vspace(g));
 	      New(new_acat, ACAT);
 	      adjust_cat(new_acat) = padjust(*style);
 	      FposCopy(fpos(new_acat), fpos(g));
@@ -1260,11 +1277,12 @@ OBJECT *enclose, BOOLEAN fcr)
 	        word_colour(z) = colour(*style);
 	        word_language(z) = language(*style);
 	        word_hyph(z) = hyph_style(*style) == HYPH_ON;
+		underline(z) = UNDER_OFF;
 	        Link(new_acat, z);
 	        New(z, GAP_OBJ);
 	        hspace(z) = hspace(g);
 	        vspace(z) = 0;
-	        underline(z) = FALSE;
+	        underline(z) = UNDER_OFF;
 	        GapCopy(gap(z), space_gap(*style));
 	        width(gap(z)) *= hspace(z);
 	        Link(new_acat, z);
@@ -1346,6 +1364,8 @@ OBJECT *enclose, BOOLEAN fcr)
 
     case HCONTRACT:
     case VCONTRACT:
+    case HLIMITED:
+    case VLIMITED:
     case HEXPAND:
     case VEXPAND:
     case ONE_COL:
@@ -1353,7 +1373,7 @@ OBJECT *enclose, BOOLEAN fcr)
     
       ETC:
       par = (type(x)==ONE_COL || type(x)==HEXPAND || type(x) == HCONTRACT ||
-	     type(x)==WIDE || type(x)==HSHIFT) ? COLM : ROWM;
+	   type(x)==HLIMITED || type(x)==WIDE || type(x)==HSHIFT) ? COLM : ROWM;
       Child(y, Down(x));
 
       /* manifest the child, propagating perp threads and suppressing pars */
@@ -1368,6 +1388,13 @@ OBJECT *enclose, BOOLEAN fcr)
       break;
 
 
+    case HSPAN:
+    case VSPAN:
+
+      ReplaceWithSplit(x, bthr, fthr);
+      break;
+
+
     case ROTATE:
 
       Child(y, Down(x));
@@ -1384,18 +1411,31 @@ OBJECT *enclose, BOOLEAN fcr)
       sparec(constraint(x)) = width(res_gap);
       DisposeChild(Down(x));
       Child(y, Down(x));
-      y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE, enclose, fcr);
+      y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE,enclose,fcr);
       ReplaceWithSplit(x, bthr, fthr);
       break;
 
 
+    case BACKGROUND:
+
+      Child(y, Down(x));
+      y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE,enclose,fcr);
+      Child(y, LastDown(x));
+      y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE,enclose,fcr);
+      ReplaceWithSplit(x, bthr, fthr);
+      break;
+
+
+    case START_HVSPAN:
+    case START_HSPAN:
+    case START_VSPAN:
     case HSCALE:
     case VSCALE:
     case HCOVER:
     case VCOVER:
 
       Child(y, Down(x));
-      y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE, enclose, fcr);
+      y = Manifest(y, env, style, nbt, nft, target, crs, ok, FALSE,enclose,fcr);
       ReplaceWithSplit(x, bthr, fthr);
       break;
 
@@ -1451,6 +1491,16 @@ OBJECT *enclose, BOOLEAN fcr)
       break;
 
 
+    case RAW_VERBATIM:
+    case VERBATIM:
+
+      Child(y, Down(x));
+      y = Manifest(y, env, style, bthr, fthr, target, crs, ok, FALSE, enclose, fcr);
+      DeleteLink(Down(x));
+      MergeNode(y, x);  x = y;
+      break;
+
+
     case CASE:
 
       x = ManifestCase(x,env,style, bthr, fthr, target, crs, ok, need_expand, enclose, fcr);
@@ -1515,6 +1565,24 @@ OBJECT *enclose, BOOLEAN fcr)
       break;
 
 
+    case CURR_FAMILY:
+    case CURR_FACE:
+
+      if( font(*style) == 0 )
+      { Error(8, 38, "no current font at this point, using %s",
+	  WARN, &fpos(x), STR_NONE);
+	res = MakeWord(WORD, STR_NONE, &fpos(x));
+      }
+      else if( type(x) == CURR_FAMILY )
+	res = MakeWord(WORD, FontFamily(font(*style)), &fpos(x));
+      else
+	res = MakeWord(WORD, FontFace(font(*style)), &fpos(x));
+      ReplaceNode(res, x);
+      DisposeObject(x);
+      x = Manifest(res, env, style, bthr, fthr, target, crs, ok, FALSE, enclose, fcr);
+      break;
+
+
     case FONT:
     case SPACE:
     case YUNIT:
@@ -1589,6 +1657,7 @@ OBJECT *enclose, BOOLEAN fcr)
       break;
 
 
+    case MELD:
     case COMMON:
     case RUMP:
 
@@ -1625,7 +1694,7 @@ OBJECT *enclose, BOOLEAN fcr)
 	New(newx2, ACAT);
         adjust_cat(newx2) = padjust(*style);
 	padjust(*style) = FALSE;
-        MoveLink(Down(x), newx2, CHILD);
+        MoveLink(NextDown(Down(x)), newx2, CHILD);
         Link(newx2, x2);
         x2 = newx2;
       }
@@ -1641,57 +1710,66 @@ OBJECT *enclose, BOOLEAN fcr)
       }
       debug1(DHY, DDD, "  manifested x2 = %s", EchoObject(x2));
      
-      /* find the point where x1 and x2 begin to differ */
-      link1 = Down(x1);
-      link2 = Down(x2);
-      while( link1 != x1 && link2 != x2 )
+      if( type(x) == MELD )
       {
-	Child(y1, link1);
-	Child(y2, link2);
-	debug1(DHY, DDD, "    y1 = %s", EchoObject(y1));
-	debug1(DHY, DDD, "    y2 = %s", EchoObject(y2));
-	if( is_word(type(y1)) && is_word(type(y2)) )
-	{
-	  if( !StringEqual(string(y1), string(y2)) )  break;
-	}
-	else if( type(y1) != type(y2) )  break;
-	link1 = NextDown(link1);
-	link2 = NextDown(link2);
+        /* if Meld, result is Meld(x1, x2) */
+        res = Meld(x1, x2);
       }
+      else
+      {
 
-      /* if COMMON, result is x1 or x2 if either ran out,             */
-      /* or else x2 (say) up to but not including link2 and prec gap  */
-      if( type(x) == COMMON )
-      { if( link2 == x2 )
-	{ res = x2;
-	}
-	else if( link1 == x1 )
-	{ res = x1;
-	}
-	else
-	{ if( link2 == Down(x2) )
-	    res = MakeWord(WORD, STR_EMPTY, &fpos(x2));
+        /* find the point where x1 and x2 begin to differ */
+        link1 = Down(x1);
+        link2 = Down(x2);
+        while( link1 != x1 && link2 != x2 )
+        {
+	  Child(y1, link1);
+	  Child(y2, link2);
+	  debug1(DHY, DDD, "    y1 = %s", EchoObject(y1));
+	  debug1(DHY, DDD, "    y2 = %s", EchoObject(y2));
+	  if( is_word(type(y1)) && is_word(type(y2)) )
+	  {
+	    if( !StringEqual(string(y1), string(y2)) )  break;
+	  }
+	  else if( type(y1) != type(y2) )  break;
+	  link1 = NextDown(link1);
+	  link2 = NextDown(link2);
+        }
+
+        /* if COMMON, result is x1 or x2 if either ran out,             */
+        /* or else x2 (say) up to but not including link2 and prec gap  */
+        if( type(x) == COMMON )
+        { if( link2 == x2 )
+	  { res = x2;
+	  }
+	  else if( link1 == x1 )
+	  { res = x1;
+	  }
 	  else
-	  { TransferLinks(PrevDown(link2), x2, x1);
+	  { if( link2 == Down(x2) )
+	      res = MakeWord(WORD, STR_EMPTY, &fpos(x2));
+	    else
+	    { TransferLinks(PrevDown(link2), x2, x1);
+	      res = x2;
+	    }
+	  }
+        }
+
+        /* if RUMP, result is x2 starting from link2 or NextDown(link2) */
+        else if( type(x) == RUMP )
+        { if( link2 == x2 )
+	    res = MakeWord(WORD, STR_EMPTY, &fpos(x2));
+	  else if( link1 == x1 )
+	  { 
+	    TransferLinks(Down(x2), NextDown(link2), x1);
 	    res = x2;
 	  }
-	}
-      }
-
-      /* if RUMP, result is x2 starting from link2 or NextDown(link2) */
-      else
-      { if( link2 == x2 )
-	  res = MakeWord(WORD, STR_EMPTY, &fpos(x2));
-	else if( link1 == x1 )
-	{ 
-	  TransferLinks(Down(x2), NextDown(link2), x1);
-	  res = x2;
-	}
-	else /* link1 != x1 */
-	{
-	  TransferLinks(Down(x2), link2, x1);
-	  res = x2;
-	}
+	  else /* link1 != x1 */
+	  {
+	    TransferLinks(Down(x2), link2, x1);
+	    res = x2;
+	  }
+        }
       }
 
       /* now res replaces x */
@@ -1726,16 +1804,51 @@ OBJECT *enclose, BOOLEAN fcr)
       break;
 
 
+    case ONE_OF:
+
+      Child(y, Down(x));
+      if( type(y) != ACAT )
+      {
+	/* child is not a sequence of choices, so ignore ONE_OF */
+	Error(8, 39, "%s ignored: no choices in right parameter", WARN,
+	  &fpos(x), KW_ONE_OF);
+        y = Manifest(y, env, style, bthr, fthr, target, crs, ok, FALSE,
+	      enclose, fcr);
+        DeleteLink(Down(x));
+        MergeNode(y, x);  x = y;
+      }
+      else
+      {
+	/* try each child in turn; result is first to find *target */
+	OBJECT target_before;
+	for( link = Down(y);  link != y;  link = NextDown(link) )
+	{
+	  Child(z, link);
+	  if( type(z) == GAP_OBJ )  continue;
+	  target_before = *target;
+	  z = Manifest(z, env, style, bthr, fthr, target, crs, ok, FALSE,
+	    enclose, fcr);
+	  if( *target != target_before )
+	    break;
+	}
+	DeleteLink(Up(z));
+	ReplaceNode(z, x);
+	DisposeObject(x);
+	x = z;
+      }
+      break;
+
+
     case NEXT:
 
       assert( Down(x) != x, "Manifest/NEXT: Down(x) == x!" );
       Child(y, Down(x));
-      debug1(DCS, D, "  Manifesting Next( %s, 1 )", EchoObject(y));
+      debug1(DCS, DD, "  Manifesting Next( %s, 1 )", EchoObject(y));
       y = Manifest(y, env, style, bthr, fthr, target, crs, FALSE, FALSE, enclose, fcr);
-      debug1(DCS, D, "  calling Next( %s, 1 )", EchoObject(y));
+      debug1(DCS, DD, "  calling Next( %s, 1 )", EchoObject(y));
       done = FALSE;
       y = Next(y, 1, &done);
-      debug2(DCS, D, "  Next(done = %s) returning %s",
+      debug2(DCS, DD, "  Next(done = %s) returning %s",
 			bool(done), EchoObject(y));
       DeleteLink(Down(x));
       MergeNode(y, x);  x = y;
@@ -1761,7 +1874,7 @@ OBJECT *enclose, BOOLEAN fcr)
       else
       { res = MakeWord(WORD, STR_NOCROSS, &fpos(x));
       }
-      debug4(DOM, D, "{ %s } %s { %s } = %s", EchoObject(y), Image(type(x)),
+      debug4(DOM, DD, "{ %s } %s { %s } = %s", EchoObject(y), Image(type(x)),
 	EchoObject(z), EchoObject(res));
       res = Manifest(res, env, style, bthr, fthr, target, crs, FALSE, FALSE, enclose, fcr);
       ReplaceNode(res, x);
@@ -1823,6 +1936,7 @@ OBJECT *enclose, BOOLEAN fcr)
       break;
 
 
+    case PLAIN_GRAPHIC:
     case GRAPHIC:
 
       debug1(DRS, DD, "  graphic style in Manifest = %s", EchoStyle(style));
@@ -1839,7 +1953,7 @@ OBJECT *enclose, BOOLEAN fcr)
     case SINCGRAPHIC:
 
       StyleCopy(save_style(x), *style);
-      debug2(DGP, D, "  manifest at %s (style %s)",
+      debug2(DGP, DD, "  manifest at %s (style %s)",
 	EchoObject(x), EchoStyle(&save_style(x)));
       Child(y, Down(x));
       y = Manifest(y, env, style, nbt, nft, &ntarget, crs, FALSE, FALSE, &nenclose, fcr);
@@ -1869,9 +1983,7 @@ OBJECT *enclose, BOOLEAN fcr)
   debug1(DOM, DDD, "down:  ", EchoObject(fthr[COLM]));
   debug1(DOM, DDD, "left:  ", EchoObject(bthr[ROWM]));
   debug1(DOM, DDD, "right: ", EchoObject(fthr[ROWM]));
-  debugcond1(DHY, D, eee, "] Manifest returning %s", EchoObject(x));
-#if DEBUG_ON
+  debugcond1(DHY, DD, eee, "] Manifest returning %s", EchoObject(x));
   depth--;
-#endif
   return x;
 } /* end Manifest */
