@@ -1,7 +1,7 @@
 /*@z11.c:Style Service:EchoStyle()@*******************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -10,7 +10,7 @@
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,14 +20,14 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z11.c                                                      */
 /*  MODULE:       Style Service                                              */
 /*  EXTERNS:      EchoStyle(), SpaceChange(), BreakChange()                  */
 /*                                                                           */
 /*****************************************************************************/
-#include "externs"
+#include "externs.h"
 
 
 #if DEBUG_ON
@@ -78,6 +78,8 @@ FULL_CHAR *EchoStyle(STYLE *style)
   { StringCat(res, AsciiToFull(":z="));
     StringCat(res, EchoLength(zunit(*style)));
   }
+  if( nobreakfirst(*style) ) StringCat(res, AsciiToFull(":NBF"));
+  if( nobreaklast(*style) ) StringCat(res, AsciiToFull(":NBL"));
   StringCat(res, AsciiToFull("]"));
   return res;
 } /* end EchoStyle */
@@ -92,7 +94,7 @@ FULL_CHAR *EchoStyle(STYLE *style)
 /*                                                                           */
 /*****************************************************************************/
 
-void changespace(STYLE *style, OBJECT x)
+static void changespace(STYLE *style, OBJECT x)
 { GAP res_gap;  unsigned gap_inc;
   assert( is_word(type(x)), "changespace: type(x)!" );
   if( beginsbreakstyle(string(x)[0]) )
@@ -122,7 +124,7 @@ void changespace(STYLE *style, OBJECT x)
       mode(space_gap(*style))  = mode(res_gap);
       width(space_gap(*style)) = gap_inc == GAP_ABS ? width(res_gap) :
 	     gap_inc == GAP_INC ? width(space_gap(*style)) + width(res_gap) :
-	     max(width(space_gap(*style)) - width(res_gap), 0);
+	     find_max(width(space_gap(*style)) - width(res_gap), 0);
     }
   }
   debug1(DSS, D, "SpaceChange returning %s", EchoStyle(style));
@@ -198,6 +200,14 @@ static void changebreak(STYLE *style, OBJECT x)
 	fill_style(*style) = FILL_OFF, display_style(*style) = DISPLAY_CENTRE;
     else if( StringEqual(string(x), STR_BREAK_RLINES) )
 	fill_style(*style) = FILL_OFF, display_style(*style) = DISPLAY_RIGHT;
+    else if( StringEqual(string(x), STR_BREAK_NOFIRST) )
+	nobreakfirst(*style) = TRUE;
+    else if( StringEqual(string(x), STR_BREAK_FIRST) )
+	nobreakfirst(*style) = FALSE;
+    else if( StringEqual(string(x), STR_BREAK_NOLAST) )
+	nobreaklast(*style) = TRUE;
+    else if( StringEqual(string(x), STR_BREAK_LAST) )
+	nobreaklast(*style) = FALSE;
     else Error(11, 5, "unknown option to %s symbol (%s)",
 	   WARN, &fpos(x), KW_BREAK, string(x));
   }
@@ -211,7 +221,7 @@ static void changebreak(STYLE *style, OBJECT x)
       mode(line_gap(*style))  = mode(res_gap);
       width(line_gap(*style)) = gap_inc == GAP_ABS ? width(res_gap) :
 	gap_inc == GAP_INC ? width(line_gap(*style)) + width(res_gap) :
-	max(width(line_gap(*style)) - width(res_gap), 0);
+	find_max(width(line_gap(*style)) - width(res_gap), 0);
     }
   }
 } /* end changebreak */
@@ -267,7 +277,7 @@ void YUnitChange(STYLE *style, OBJECT x)
   else
   { if( gap_inc == GAP_ABS ) yunit(*style) = width(res_gap);
     else if( gap_inc == GAP_INC ) yunit(*style) += width(res_gap);
-    else yunit(*style) = max(yunit(*style) - width(res_gap), 0);
+    else yunit(*style) = find_max(yunit(*style) - width(res_gap), 0);
   }
 } /* end YUnitChange */
 
@@ -289,6 +299,6 @@ void ZUnitChange(STYLE *style, OBJECT x)
   else
   { if( gap_inc == GAP_ABS ) zunit(*style) = width(res_gap);
     else if( gap_inc == GAP_INC ) zunit(*style) += width(res_gap);
-    else zunit(*style) = max(zunit(*style) - width(res_gap), 0);
+    else zunit(*style) = find_max(zunit(*style) - width(res_gap), 0);
   }
 } /* end ZUnitChange */
