@@ -1,7 +1,7 @@
 /*@z19.c:Galley Attaching:DetachGalley()@*************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -10,7 +10,7 @@
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,14 +20,14 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z19.c                                                      */
 /*  MODULE:       Galley Attaching                                           */
 /*  EXTERNS:      SearchGalley(), AttachGalley(), DetachGalley()             */
 /*                                                                           */
 /*****************************************************************************/
-#include "externs"
+#include "externs.h"
 
 
 /*****************************************************************************/
@@ -124,7 +124,7 @@ void DetachGalley(OBJECT hd)
 OBJECT SearchGalley(OBJECT start, OBJECT sym, BOOLEAN forwards,
 BOOLEAN subgalleys, BOOLEAN closures, BOOLEAN input)
 { OBJECT y, res, z, zlink, link;
-  debug5(DGA, DDD, "[ SearchGalley(start, %s, %s, %s, %s, %s)", SymName(sym),
+  debug5(DGA, DD, "[ SearchGalley(start, %s, %s, %s, %s, %s)", SymName(sym),
 	forwards ? "fwd" : "back", subgalleys ? "subgalleys" : "nosubgalleys",
 	closures ? "closures" : "noclosures", input ? "input" : "noinput");
   assert( type(start) == LINK || type(start) == HEAD, "SearchGalley: start!" );
@@ -138,20 +138,21 @@ BOOLEAN subgalleys, BOOLEAN closures, BOOLEAN input)
       case UNATTACHED:
       case RECEIVING:
 	
-        debug1(DGA, DDD, "  examining %s", EchoIndex(y));
+        debug1(DGA, DD, "  examining %s", EchoIndex(y));
 	if( subgalleys )
 	for( zlink = Down(y); zlink!=y && res==nilobj; zlink=NextDown(zlink) )
 	{ Child(z, zlink);
 	  res = SearchGalley(z, sym, TRUE, TRUE, TRUE, input);
 	}
-	if( !res && input && type(y)==RECEIVING && actual(actual(y))==InputSym )
+	if( res == nilobj && input && type(y) == RECEIVING &&
+	    actual(actual(y)) == InputSym )
 	  res = y;
 	break;
 
 
       case RECEPTIVE:
 	
-        debug1(DGA, DDD, "  examining %s", EchoIndex(y));
+        debug1(DGA, DD, "  examining %s", EchoIndex(y));
 	if( closures && type(actual(y)) == CLOSURE
 		     && SearchUses(actual(actual(y)), sym) )  res = y;
 	else if( input && actual(actual(y)) == InputSym )  res = y;
@@ -165,7 +166,7 @@ BOOLEAN subgalleys, BOOLEAN closures, BOOLEAN input)
     }
     link = forwards ? NextDown(link) : PrevDown(link);
   }
-  debug1(DGA, DDD, "] SearchGalley returning %s", EchoIndex(res));
+  debug1(DGA, DD, "] SearchGalley returning %s", EchoIndex(res));
   return res;
 } /* end SearchGalley */
 
@@ -342,6 +343,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	EchoObject(target), EchoConstraint(&c));
       if( !FitsConstraint(0, 0, c) )
       { debug0(DGA, D, "  reject: target_galley horizontal constraint is -1");
+	y = nilobj;
         goto REJECT;
       }
     }
@@ -364,6 +366,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
     debug1(DGA, DD, "  SizeGalley tg_inners: %s", DebugInnersNames(tg_inners));
     if( recs != nilobj )  ExpandRecursives(recs);
     dest = actual(dest_index);
+    if( underline(dest) == UNDER_UNDEF )  underline(dest) = UNDER_OFF;
 
     /* verify that hd satisfies any horizontal constraint on dest */
     if( dim == ROWM )
@@ -375,6 +378,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
       assert( constrained(c), "AttachGalley: dest unconstrained!" );
       if( !FitsConstraint(0, 0, c) )
       { debug0(DGA, D, "  reject: hd horizontal constraint is -1");
+	y = nilobj;
         goto REJECT;
       }
     }
@@ -502,6 +506,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 
 	case GAP_OBJ:
 
+	  underline(y) = underline(dest);
 	  if( !join(gap(y)) )  seen_nojoin(hd) = TRUE;
 	  break;
 
@@ -512,6 +517,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	case NULL_CLOS:
 	case PAGE_LABEL:
 
+	  underline(y) = underline(dest);
 	  break;
 
 
@@ -529,13 +535,22 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	case VCOVER:
 	case HCONTRACT:
 	case VCONTRACT:
+	case HLIMITED:
+	case VLIMITED:
 	case HEXPAND:
 	case VEXPAND:
+	case START_HVSPAN:
+	case START_HSPAN:
+	case START_VSPAN:
+	case HSPAN:
+	case VSPAN:
 	case ROTATE:
+	case BACKGROUND:
 	case SCALE:
 	case KERN_SHRINK:
 	case INCGRAPHIC:
 	case SINCGRAPHIC:
+	case PLAIN_GRAPHIC:
 	case GRAPHIC:
 	case ACAT:
 	case HCAT:
@@ -544,6 +559,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	case COL_THR:
 	    
 
+	  underline(y) = underline(dest);
 	  if( dim == ROWM )
 	  {
 	    /* make sure y is not joined to a target below (vertical only) */
@@ -872,7 +888,8 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
     REJECT:
 	
       /* reject first component */
-      debug1(DGA, D, "  reject %s", EchoObject(y));
+      /* debug1(DGA, D, "  reject %s", EchoObject(y)); */
+      debug0(DGA, D, "  reject first component");
       LeaveErrorBlock(TRUE);
       debug0(DYY, D, "] LeaveErrorBlock(TRUE) (REJECT)");
       if( tg_inners != nilobj )  DisposeObject(tg_inners), tg_inners = nilobj;
@@ -938,7 +955,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
 	StyleCopy(save_style(junk), save_style(dest));
 	adjust_cat(junk) = padjust(save_style(junk));
       }
-      Promote(hd, link == hd ? hd : NextDown(link), dest_index);
+      Promote(hd, link == hd ? hd : NextDown(link), dest_index, TRUE);
 
       /* move target_galley into target */
       /* nb Interpose must be done after all AdjustSize calls */
@@ -946,7 +963,7 @@ int AttachGalley(OBJECT hd, OBJECT *inners, OBJECT *suspend_pt)
       {	Child(z, LastDown(target_galley));
 	Interpose(target, VCAT, z, z);
       }
-      Promote(target_galley, target_galley, target_index);
+      Promote(target_galley, target_galley, target_index, TRUE);
       DeleteNode(target_galley);
       assert(Down(target_index)==target_index, "AttachGalley: target_ind");
       if( blocked(target_index) )  blocked(dest_index) = TRUE;
