@@ -1,7 +1,7 @@
 /*@z23.c:Galley Printer:ScaleFactor()@****************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -10,7 +10,7 @@
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,21 +20,18 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z23.c                                                      */
 /*  MODULE:       Galley Printer                                             */
 /*  EXTERNS:      FixAndPrintObject()                                        */
 /*                                                                           */
 /*****************************************************************************/
-#include "externs"
+#include "externs.h"
 #define	NO_SUPPRESS	FALSE
 #define	SUPPRESS	TRUE
 #define word_equal(x, str)  (is_word(type(x)) && StringEqual(string(x), str))
 
-#define CountChild(y, link, i)						\
-for( y=pred(link, PARENT), i=1; type(y)==LINK;  y = pred(y, PARENT), i++ )
-
 
 /*****************************************************************************/
 /*                                                                           */
@@ -128,11 +125,13 @@ static FULL_LENGTH FindAdjustIncrement(OBJECT x, FULL_LENGTH frame_size,int dim)
 
 void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
   FULL_LENGTH xf, int dim, BOOLEAN suppress, FULL_LENGTH pg, int count)
-{ OBJECT y, link, prev, g, uplink, z, face;
+{ OBJECT y, link, prev, g, uplink, z, face, thr;
   FULL_LENGTH mk, ymk, frame_size, back_edge, yb, yf, inc, f;
   int i; float scale_factor;  BOOLEAN jn;
-  debug6(DGP, DD, "[ FixAndPrintObject(%s, %s, %s,%s, %s, %s, pg )",
-    Image(type(x)), EchoLength(xmk), EchoLength(xb), EchoLength(xf),dimen(dim),
+  debug7(DGP, DD, "[ FixAndPrintObject(%s %s, %s, %s,%s, %s, %s, pg )",
+    Image(type(x)),
+    ((type(x) == WORD || type(x) == QWORD) ? string(x) : STR_EMPTY),
+    EchoLength(xmk), EchoLength(xb), EchoLength(xf),dimen(dim),
     (suppress == SUPPRESS ? "suppress" : "no_suppress"));
   debug2(DGP, DD, "  size(x) = %s,%s;  x =",
     EchoLength(back(x, dim)), EchoLength(fwd(x, dim)));
@@ -161,6 +160,52 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
       break;
 
 
+    case START_HVSPAN:
+    case START_HSPAN:
+    case START_VSPAN:
+
+      CountChild(y, DownDim(x, dim), count);
+      if( type(y) == HSPANNER || type(y) == VSPANNER )
+      {
+        Child(z, Down(y));
+	Parent(thr, UpDim(x, dim));
+        debug7(DGP, DD, "  calling SPAN %s(xmk %s, x %s,%s, cons %s, z %s,%s)",
+	    dimen(dim), EchoLength(xmk),
+	    EchoLength(back(x, dim)), EchoLength(fwd(x, dim)),
+	    EchoConstraint(&constraint(y)),
+	    EchoLength(back(z, dim)), EchoLength(fwd(z, dim)));
+	/* ***
+	f = find_max(xf, fwd(z,dim));
+        FixAndPrintObject(z, xmk - back(thr, dim) + back(z, dim), back(z, dim),
+	  find_max(f, bfc(constraint(y)) - back(z, dim)),
+	  dim, FALSE, pg, 1);
+	*** */
+	debug5(DGP, DD, "  calling FAPO from %s (y = %s, bfc = %s, z = %s,%s",
+	  Image(type(x)), Image(type(y)), EchoLength(bfc(constraint(y))),
+	  EchoLength(back(z, dim)), EchoLength(fwd(z, dim)));
+	/* ***
+        FixAndPrintObject(z, xmk - back(thr, dim) + back(z, dim), back(z, dim),
+	  bfc(constraint(y)) - back(z, dim), dim, FALSE, pg, 1);
+	*** */
+        FixAndPrintObject(z, xmk - back(thr, dim) + back(z, dim), back(z, dim),
+	  find_max(fwd(z, dim), bfc(constraint(y)) - back(z, dim)),
+	  dim, FALSE, pg, 1);
+      }
+      else
+      {
+        FixAndPrintObject(y, xmk, xb, xf, dim, suppress, pg, count);
+	back(x, dim) = back(y, dim);  fwd(x, dim) = fwd(y, dim);
+      }
+      break;
+
+
+    case HSPAN:
+    case VSPAN:
+
+      /* nothing to print, spanner beneath is already done */
+      break;
+
+
     case WORD:
     case QWORD:
     
@@ -181,11 +226,16 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
       }
       else
       {
-	debug3(DGP, DD, "  FAPO %s %s (underline = %s)", Image(type(x)),
-	  string(x), bool(underline(x)));
+	debug4(DGP, D, /* underline(x) == UNDER_UNDEF, */
+	  "  FAPO %s %s %s (underline = %s)", EchoFilePos(&fpos(x)),
+	  Image(type(x)),
+	  string(x), underline(x) == UNDER_OFF ? "UNDER_OFF" :
+		     underline(x) == UNDER_ON  ? "UNDER_ON" : "UNDER_UNDEF");
+	assert( underline(x) == UNDER_OFF || underline(x) == UNDER_ON,
+	  "FixAndPrintObject: underline(x)!" );
 	if( string(x)[0] != '\0' )
 	{ PrintWord(x, word_save_mark(x), pg - xmk);
-	  if( underline(x) )
+	  if( underline(x) == UNDER_ON )
 	  {
 	    FontWordSize(x);  /* to restore fwd(x, COLM) */
 	    PrintUnderline(word_font(x), word_save_mark(x),
@@ -222,14 +272,14 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 	/* work out the size of the shift depending on the units */
 	f = FindShift(x, y, dim);
 	ymk = xmk - f;
-	yb = max(0, xb - f);
-	yf = max(0, xf + f);
+	yb = find_max(0, xb - f);
+	yf = find_max(0, xf + f);
 	FixAndPrintObject(y, ymk, yb, yf, dim, suppress, pg, count);
 
 	/* recalculate the size of x as in MinSize */
 	f = FindShift(x, y, dim);
-	back(x, dim) = min(MAX_FULL_LENGTH, max(0, back(y, dim) + f));
-	fwd(x, dim)  = min(MAX_FULL_LENGTH, max(0, fwd(y, dim)  - f));
+	back(x, dim) = find_min(MAX_FULL_LENGTH, find_max(0, back(y, dim) + f));
+	fwd(x, dim)  = find_min(MAX_FULL_LENGTH, find_max(0, fwd(y, dim)  - f));
       }
       else
       {	FixAndPrintObject(y, xmk, xb, xf, dim, suppress, pg, count);
@@ -256,6 +306,8 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 
     case ONE_COL:
     case ONE_ROW:
+    case HLIMITED:
+    case VLIMITED:
     case HEXPAND:
     case VEXPAND:
     
@@ -283,6 +335,7 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 
 
 	case POSTSCRIPT:
+	case PDF:
 
           if( dim == COLM )
             FixAndPrintObject(y, xmk, xb, xf, dim, NO_SUPPRESS, pg, count);
@@ -315,6 +368,7 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 
 
 	case POSTSCRIPT:
+	case PDF:
 
           if( dim == COLM )
           { save_mark(x) = xmk;
@@ -358,6 +412,7 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 
 
 	case POSTSCRIPT:
+	case PDF:
 
           if( dim == COLM )
           { assert( bc(constraint(x)) > 0, "FAPO: horizontal scale factor!" );
@@ -398,6 +453,17 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
       break;
 
 
+    case BACKGROUND:
+ 
+      /* this object has the size of its second child; but its first */
+      /* child gets printed too, in the same space                   */
+      CountChild(y, Down(x), count);
+      FixAndPrintObject(y, xmk, xb, xf, dim, suppress, pg, count);
+      CountChild(y, LastDown(x), count);
+      FixAndPrintObject(y, xmk, xb, xf, dim, suppress, pg, count);
+      break;
+
+
     case ROTATE:
     
       CountChild(y, Down(x), count);
@@ -414,6 +480,7 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 
 
 	case POSTSCRIPT:
+	case PDF:
 
           if( dim == COLM )
           { CONSTRAINT colc, rowc, yc;
@@ -440,6 +507,54 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
       break;
 
 
+    case PLAIN_GRAPHIC:
+
+      CountChild(y, LastDown(x), count);
+      switch( BackEnd )
+      {
+
+	case POSTSCRIPT:
+	case PDF:
+
+          FixAndPrintObject(y, xmk, xb, xf, dim, suppress, pg, count);
+	  break;
+
+
+	case PLAINTEXT:
+
+          if( dim == COLM )
+          {
+	    back(x, dim) = xb;
+	    fwd(x, dim)  = xf;
+	    save_mark(x) = xmk - back(x, dim);
+	    debug2(DGP, DD, "PLAIN_GRAPHIC COLM storing size %s, %s",
+	      EchoLength(back(x, dim)), EchoLength(fwd(x, dim)));
+            FixAndPrintObject(y, xmk, xb, xf, dim, suppress, pg, count);
+          }
+          else
+          { OBJECT tmp, pre, post;
+            Child(tmp, Down(x));
+            if( type(tmp) == VCAT )
+            { Child(pre, Down(tmp));
+              Child(post, LastDown(tmp));
+            }
+            else pre = tmp, post = nilobj;
+	    back(x, dim) = xb;
+	    fwd(x, dim)  = xf;
+            PrintPlainGraphicObject(pre, save_mark(x),
+	      pg - (xmk - back(x, dim)), x);
+            FixAndPrintObject(y, xmk, xb, xf, dim, suppress, pg, count);
+            if( post != nilobj )
+	      PrintPlainGraphicObject(post, save_mark(x),
+		pg - (xmk - back(x, dim)), x);
+          }
+	  break;
+
+      } /* end switch */
+      back(x, dim) = xb;  fwd(x, dim) = xf;
+      break;
+
+
     case GRAPHIC:
     
       CountChild(y, LastDown(x), count);
@@ -452,6 +567,7 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 
 
 	case POSTSCRIPT:
+	case PDF:
 
           if( dim == COLM )
           {
@@ -515,11 +631,12 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 
 
 	case POSTSCRIPT:
+	case PDF:
 
           if( dim == COLM )
 	  { save_mark(x) = xmk;
 	    if( incgraphic_ok(x) )
-	    { debug2(DGP, D, "  %s (style %s)",
+	    { debug2(DGP, DD, "  %s (style %s)",
 		EchoObject(x), EchoStyle(&save_style(x)));
 	      face = finfo[font(save_style(x))].original_font;
 	      if( font_page(face) < font_curr_page )
@@ -551,21 +668,69 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 
       if( (type(x) == VCAT) == (dim == ROWM) )
       { 
-	/* find adjustment increment if required */
-	frame_size = xb + xf;
-	if( adjust_cat(x) && !suppress )
-	  inc = FindAdjustIncrement(x, frame_size, dim);
-	else inc = 0;
-
 	FirstDefinite(x, link, prev, jn);
 	if( link != x )
-	{ back_edge = xmk - back(x, dim);
-	  mk = back_edge + back(prev, dim);
+	{
+
+	  /* handle the special case of a 0rt gap at the beginning (left  */
+	  /* justify) by converting it to 0ie but increasing fwd(prev)    */
+	  /* to the max. possible                                         */
 	  NextDefiniteWithGap(x, link, y, g, jn);
+	  if( link != x && mode(gap(g)) == TAB_MODE &&
+	      units(gap(g)) == AVAIL_UNIT && width(gap(g)) == 0 )
+	  {
+	    debug2(DGP, DD, "  FAPO-CAT converting 0rt (back(x, dim) %s, xb %s)",
+	      EchoLength(back(x, dim)), EchoLength(xb));
+	    fwd(prev, dim) += xb - back(x, dim);
+	    back(x, dim) = xb;
+	    mode(gap(g)) = EDGE_MODE;
+	    units(gap(g)) = FIXED_UNIT;
+	  }
+	  FirstDefinite(x, link, prev, jn);
+
+	  /* the frame size is the total width actually available */
+	  frame_size = back(x, dim) + xf;
+
+	  /* back_edge is where the first element begins */
+	  back_edge = xmk - back(x, dim);
+
+	  /* inc is the adjust increment, used when adjusting gaps */
+	  if( adjust_cat(x) && !suppress )
+	    inc = FindAdjustIncrement(x, frame_size, dim);
+	  else inc = 0;
+
+	  debug6(DGP, DD, "[ FAPO-CAT %s (%s,%s): xmk %s, xb %s, xf %s",
+	    Image(type(x)), EchoLength(back(x, dim)), EchoLength(fwd(x, dim)),
+	    EchoLength(xmk), EchoLength(xb), EchoLength(xf));
+
+	  mk = back_edge + back(prev, dim);
+	  debug4(DGP, DD, "  FAPO-CAT back_edge %s, mk %s, framesize %s, inc %s",
+	    EchoLength(back_edge), EchoLength(mk), EchoLength(frame_size),
+	    EchoLength(inc));
+
+	  NextDefiniteWithGap(x, link, y, g, jn);
+
 	  while( link != x )
 	  {
-	    FixAndPrintObject(prev, mk, back(prev, dim), fwd(prev, dim) + inc,
-	      dim, NO_SUPPRESS, pg, count);
+	    if( mode(gap(g)) == TAB_MODE && units(gap(g)) == AVAIL_UNIT &&
+		width(gap(g))==FR )
+	    {
+	      /* object is followed by 1rt gap, give it full space to print */
+	      debug5(DGP,D,"  FAPO (a) calling FAPO(%s, %s, %s, max(%s, %s))",
+	        Image(type(prev)), EchoLength(mk), EchoLength(back(prev, dim)),
+		EchoLength(fwd(prev, dim)), EchoLength(xmk+xf-mk-size(y,dim)));
+	      FixAndPrintObject(prev, mk, back(prev, dim),
+		find_max(fwd(prev, dim), xmk+xf-mk - size(y, dim)),
+		dim, NO_SUPPRESS, pg, count);
+	    }
+	    else
+	    {
+	      debug5(DGP, DD, "  FAPO-CAT (b) calling FAPO(%s, %s, %s, %s+%s)",
+	        Image(type(prev)), EchoLength(mk), EchoLength(back(prev, dim)),
+		EchoLength(fwd(prev, dim)), EchoLength(inc));
+	      FixAndPrintObject(prev, mk, back(prev, dim), fwd(prev, dim) + inc,
+	        dim, NO_SUPPRESS, pg, count);
+	    }
 	    /* NB fwd(prev, dim) may be changed by the call to FAPO */
 	    mk += ActualGap(fwd(prev, dim), back(y, dim), fwd(y, dim), &gap(g),
 		    frame_size, mk - back_edge);
@@ -573,16 +738,27 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 	    NextDefiniteWithGap(x, link, y, g, jn);
 	  }
 	  if( suppress )
+	  {
+	    debug4(DGP, DD, "  FAPO-CAT (c) calling FAPO(%s, %s, %s, %s)",
+	      Image(type(prev)), EchoLength(mk), EchoLength(back(prev, dim)),
+	      EchoLength(fwd(prev, dim)));
 	    FixAndPrintObject(prev, mk, back(prev, dim), fwd(prev, dim),
 	      dim, NO_SUPPRESS, pg, count);
+	  }
 	  else
+	  {
+	    debug5(DGP, DD,"  FAPO-CAT (d) calling FAPO(%s, %s, %s, max(%s, %s))",
+	      Image(type(prev)), EchoLength(mk), EchoLength(back(prev, dim)),
+	      EchoLength(fwd(prev, dim)), EchoLength(xmk + xf - mk));
 	    FixAndPrintObject(prev, mk, back(prev,dim),
-	      max(fwd(prev, dim), back_edge+frame_size-mk),
+	      find_max(fwd(prev, dim), xmk + xf - mk),
 	      dim, NO_SUPPRESS, pg, count);
-	  back(x, dim) = max(back(x, dim), xb);
+	  }
+	  back(x, dim) = find_max(back(x, dim), xb);
 	  fwd(x, dim) = mk + fwd(prev, dim) - back_edge - back(x, dim);
 	}
 	else back(x, dim) = xb, fwd(x, dim) = xf;
+	debug0(DGP, DD, "] FAPO-CAT returning.");
       }
       else
       { OBJECT start_group, zlink, m;  BOOLEAN dble_found;
@@ -598,7 +774,8 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 	  m = y;
 	  start_group = link;
 	  dble_found = !jn;
-	  debug2(DGP, DD, "  starting first group: b = %s, f = %s",
+	  debug4(DGP, DD, "  starting first group %s (%sdbl_found): b = %s, f = %s",
+	    Image(type(y)), dble_found ? "" : "not ",
 	    EchoLength(b), EchoLength(f));
 	
 	  NextDefiniteWithGap(x, link, y, g, jn);
@@ -617,9 +794,9 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 		if( !is_definite(type(z)) || z == m )  continue;
 		FixAndPrintObject(z, xmk + b, b, xf - b, dim,
 		  SUPPRESS, pg, count);
-		b = max(b, back(z, dim));  f = max(f, fwd(z, dim));
+		b = find_max(b, back(z, dim));  f = find_max(f, fwd(z, dim));
 	      }
-	      dlen = max(dlen, b + f);
+	      dlen = find_max(dlen, b + f);
 	      dble_found = TRUE;
 	      start_group = nilobj;
 
@@ -634,8 +811,8 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 	    else
 	    {
 	      /* continue with current group */
-	      b = max(b, back(y, dim));
-	      f = max(f, fwd(y, dim));
+	      b = find_max(b, back(y, dim));
+	      f = find_max(f, fwd(y, dim));
 	      if( fwd(y, dim) > fwd(m, dim) )  m = y;
 	      debug2(DGP, DD, "  continuing group: b = %s, f = %s",
 		EchoLength(b), EchoLength(f));
@@ -656,9 +833,9 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 	    { CountChild(z, zlink, count);
 	      if( !is_definite(type(z)) || z == m )  continue;
 	      FixAndPrintObject(z, xmk+b, b, xf - b, dim, SUPPRESS, pg, count);
-	      b = max(b, back(z, dim));  f = max(f, fwd(z, dim));
+	      b = find_max(b, back(z, dim));  f = find_max(f, fwd(z, dim));
 	    }
-	    dlen = max(dlen, b + f);
+	    dlen = find_max(dlen, b + f);
 	    back(x, dim) = 0;  fwd(x, dim) = dlen;
 	  }
 	  else
@@ -672,7 +849,7 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 	    { CountChild(z, zlink, count);
 	      if( !is_definite(type(z)) || z == m )  continue;
 	      FixAndPrintObject(z, xmk, xb, xf, dim, SUPPRESS, pg, count);
-	      b = max(b, back(z, dim));  f = max(f, fwd(z, dim));
+	      b = find_max(b, back(z, dim));  f = find_max(f, fwd(z, dim));
 	    }
 	    back(x, dim) = b;  fwd(x, dim) = f;
 	  }
@@ -743,184 +920,262 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 	}
 	actual_size = mk + fwd(prev, dim) - back_edge;
 
-
 	/*********************************************************************/
 	/*                                                                   */
-	/*  The line may be displayed in one of four ways:  centred, right-  */
-	/*  justified, adjusted, or none of the above (i.e. left justified). */
-	/*  An overfull line is always adjusted; otherwise, the line will    */
-	/*  be centred or right justified if the display style asks for it;  */
-	/*  otherwise, the line will be adjusted if adjust_cat(x) == TRUE    */
-	/*  (i.e. there is an enclosing @PAdjust) or if the display style is */
-	/*  DO_ADJUST (meaning that this line is one of a paragraph set in   */
-	/*  the adjust or outdent break style, other than the last line);    */
-	/*  otherwise, the line is left justified.                           */
-	/*                                                                   */
-	/*  The second step is to decide which of these four cases holds     */
-	/*  for this line, and to record the decision in these variables:    */
-	/*                                                                   */
-	/*    will_adjust      TRUE if the adjusted style applies; in this   */
-	/*                     case, variables adjust_inc and inc will be    */
-	/*                     set to the appropriate adjustment value;      */
-	/*                                                                   */
-	/*    adjust_indent    If centring or right justification applies,   */
-	/*                     the indent to produce this, else zero.        */
-	/*                                                                   */
-	/*  NB adjust_inc may be negative, if the optimal paragraph breaker  */
-	/*  has chosen to shrink some gaps.                                  */
+	/*  It is possible that the line cannot be displayed in any          */
+	/*  reasonable way, because the paragraph breaker was forced to      */
+	/*  produce an overfull line.  In this case, actual_size will        */
+	/*  exceed frame_size and there will be no adjustable gaps.  The     */
+	/*  solution is to horizontally scale the line if possible, or       */
+	/*  else to not print it at all.                                     */
 	/*                                                                   */
 	/*********************************************************************/
 
-	if( actual_size > frame_size )
-	{ adjust_cat(x) = TRUE;
-	  adjust_indent = 0;
-	}
-	else switch( display_style(save_style(x)) )
-	{
-	  case DO_ADJUST:	adjust_cat(x) = TRUE;
+	if( actual_size > frame_size && adjustable_gaps == 0 )
+	{ 
+	  /* can't be fixed by adjustment, so scale the line or delete it */
+	  CONSTRAINT c;
+	  SetConstraint(c, 0, frame_size, frame_size);
+	  fwd(x, dim) = actual_size;
+	  debug2(DGP, DD, "  oversize, actual_size = %s, frame_size = %s",
+	      EchoLength(actual_size), EchoLength(frame_size));
+	  if( BackEnd != PLAINTEXT && InsertScale(x, &c) )
+	  {
+	    /* the problem has just been fixed, by inserting a @Scale above x */
+	    OBJECT prnt;
+	    Parent(prnt, Up(x));
+	    Child(y, Down(x));
+	    if( actual_size - frame_size < 1 * PT )
+	    {
+	      /* the correction is probably due to roundoff error, and */
+	      /* anyway is too small to print an error message about   */
+	    }
+	    else if( Down(x) == LastDown(x) && is_word(type(y)) )
+	    {
+	      Error(23, 3, "word %s horizontally scaled by factor %.2f (too wide for %s paragraph)",
+		WARN, &fpos(y), string(y), (float) bc(constraint(prnt)) / SF,
+		EchoLength(frame_size));
+	    }
+	    else
+	    {
+	      Error(23, 4, "%s object horizontally scaled by factor %.2f (too wide for %s paragraph)",
+		WARN, &fpos(x), EchoLength(size(x, COLM)),
+		(float) bc(constraint(prnt)) / SF, EchoLength(frame_size));
+	    }
+	    FixAndPrintObject(prnt, xmk, back(prnt, dim), fwd(prnt, dim), dim,
+	      NO_SUPPRESS, pg, count);
+	  }
+	  else
+	  {
+	    /* fix the problem by refraining from printing the line */
+	    if( size(x, COLM) <= 0 )
+	      Error(23, 5, "oversize object has size 0 or less", INTERN, &fpos(x));
+	    Child(y, Down(x));
+	    if( Down(x) == LastDown(x) && is_word(type(y)) )
+	    { Error(23, 6, "word %s deleted (too wide for %s paragraph)",
+		WARN, &fpos(y), string(y), EchoLength(frame_size));
+	    }
+	    else
+	    { Error(23, 7, "%s object deleted (too wide for %s paragraph)",
+		WARN, &fpos(x), EchoLength(size(x, COLM)), EchoLength(frame_size));
+	    }
+
+	    /* delete and dispose every child of x */
+	    while( Down(x) != x )
+	      DisposeChild(Down(x));
+	    y = MakeWord(WORD, STR_EMPTY, &fpos(x));
+	    Link(x, y);
+	    back(y, COLM) = fwd(y, COLM) = 0;
+	    back(y, ROWM) = fwd(y, ROWM) = 0;
+	  }
+        }
+        else
+        {
+
+	  /*********************************************************************/
+	  /*                                                                   */
+	  /*  The line may be displayed in one of four ways:  centred, right-  */
+	  /*  justified, adjusted, or none of the above (i.e. left justified). */
+	  /*  An overfull line is always adjusted; otherwise, the line will    */
+	  /*  be centred or right justified if the display style asks for it;  */
+	  /*  otherwise, the line will be adjusted if adjust_cat(x) == TRUE    */
+	  /*  (i.e. there is an enclosing @PAdjust) or if the display style is */
+	  /*  DO_ADJUST (meaning that this line is one of a paragraph set in   */
+	  /*  the adjust or outdent break style, other than the last line);    */
+	  /*  otherwise, the line is left justified.                           */
+	  /*                                                                   */
+	  /*  The second step is to decide which of these four cases holds     */
+	  /*  for this line, and to record the decision in these variables:    */
+	  /*                                                                   */
+	  /*    will_adjust      TRUE if the adjusted style applies; in this   */
+	  /*                     case, variables adjust_inc and inc will be    */
+	  /*                     set to the appropriate adjustment value;      */
+	  /*                                                                   */
+	  /*    adjust_indent    If centring or right justification applies,   */
+	  /*                     the indent to produce this, else zero.        */
+	  /*                                                                   */
+	  /*  NB adjust_inc may be negative, if the optimal paragraph breaker  */
+	  /*  has chosen to shrink some gaps.                                  */
+	  /*                                                                   */
+	  /*********************************************************************/
+
+	  if( actual_size > frame_size )
+	  { 
+	    assert( adjustable_gaps > 0, "FAPO: adjustable_gaps!" );
+	    adjust_cat(x) = TRUE;
+	    adjust_indent = 0;
+	  }
+	  else switch( display_style(save_style(x)) )
+	  {
+	    case DO_ADJUST:	adjust_cat(x) = TRUE;
 				adjust_indent = 0;
 				break;
 	
-	  case DISPLAY_CENTRE:	adjust_cat(x) = FALSE;
+	    case DISPLAY_CENTRE: adjust_cat(x) = FALSE;
 				adjust_indent = (frame_size - actual_size)/2;
 				debug1(DGP, DD, "cdisp %s", EchoObject(x));
 				break;
 
-	  case DISPLAY_RIGHT:	adjust_cat(x) = FALSE;
+	    case DISPLAY_RIGHT:	adjust_cat(x) = FALSE;
 				adjust_indent = frame_size - actual_size;
 				debug1(DGP, DD, "rdisp %s", EchoObject(x));
 				break;
 
-	  default:		/* leave adjust_cat(x) as is */
+	    default:		/* leave adjust_cat(x) as is */
 				adjust_indent = 0;
 				break;
-	}
+	  }
 
-	debug2(DGP, DD, "ACAT %s %s",
-	  EchoStyle(&save_style(x)), EchoObject(x));
-	debug2(DGP, DD, "frame_size = %s, actual_size = %s",
-	  EchoLength(frame_size), EchoLength(actual_size));
+	  debug2(DGP, DD, "ACAT %s %s",
+	    EchoStyle(&save_style(x)), EchoObject(x));
+	  debug2(DGP, DD, "frame_size = %s, actual_size = %s",
+	    EchoLength(frame_size), EchoLength(actual_size));
 
-	if( adjust_cat(x) && adjustable_gaps > 0  )
-	{ will_adjust = TRUE;
-	  adjust_inc = (frame_size - actual_size) / adjustable_gaps;
-	  inc = max(adjust_inc, 0);
-	  gaps_sofar = 0;	/* number of gaps adjusted so far */
-	  adjust_sofar = 0;	/* total width of adjustments so far */
-	  debug2(DGP, DD, "will_adjust: adjustable_gaps = %d, adjust_inc = %s",
-	    adjustable_gaps, EchoLength(adjust_inc));
-	}
-	else will_adjust = FALSE;
+	  if( adjust_cat(x) && adjustable_gaps > 0  )
+	  { will_adjust = TRUE;
+	    adjust_inc = (frame_size - actual_size) / adjustable_gaps;
+	    inc = find_max(adjust_inc, 0);
+	    gaps_sofar = 0;	/* number of gaps adjusted so far */
+	    adjust_sofar = 0;	/* total width of adjustments so far */
+	    debug2(DGP, DD, "will_adjust: adjustable_gaps = %d, adjust_inc = %s",
+	      adjustable_gaps, EchoLength(adjust_inc));
+	  }
+	  else will_adjust = FALSE;
 
 
-	/*********************************************************************/
-	/*                                                                   */
-	/*  The third and final step is to traverse x, fixing subobjects.    */
-	/*  Variable adjusting is true while adjusting is occurring.         */
-	/*                                                                   */
-	/*********************************************************************/
+	  /*********************************************************************/
+	  /*                                                                   */
+	  /*  The third and final step is to traverse x, fixing subobjects.    */
+	  /*  Variable adjusting is true while adjusting is occurring.         */
+	  /*                                                                   */
+	  /*********************************************************************/
+
+	  underlining = FALSE;
+	  adjusting = will_adjust && last_bad_gap == nilobj;
+	  FirstDefinite(x, link, y, jn);
+	  prev = y;
+	  mk = xmk - back(x, dim) + back(y, dim) + adjust_indent;
+	  NextDefiniteWithGap(x, link, y, g, jn);
+	  while( link != x )
+	  {
+	    /* check for underlining */
+	    if( underline(prev) == UNDER_ON )
+	    {
+	      debug3(DGP, DD, "  FAPO/ACAT1 underline() := %s for %s %s",
+	        bool(FALSE), Image(type(prev)), EchoObject(prev));
+	      underline(prev) = UNDER_OFF;
+	      if( !underlining )
+	      {
+	        /* underlining begins here */
+	        underlining = TRUE;
+	        debug2(DGP, DD, "underlining begins at %s %s",
+		  Image(type(prev)), EchoObject(prev));
+	        underline_font = is_word(type(prev)) ? word_font(prev) :
+		    font(save_style(x));
+	        underline_xstart = mk - back(prev, dim);
+	      }
+	      if( underline(g) == UNDER_OFF )
+	      {
+	        /* underlining ends here */
+	        debug2(DGP, DD, "underlining ends at %s %s",
+		  Image(type(prev)), EchoObject(prev));
+	        New(urec, UNDER_REC);
+	        back(urec, COLM) = underline_xstart;
+	        fwd(urec, COLM) = mk + fwd(prev, dim);
+	        back(urec, ROWM) = underline_font;
+	        underlining = FALSE;
+	        Link(Up(prev), urec);
+	      }
+	    }
+
+	    /* fix previous definite now we know it is not the last one  */
+	    if( adjusting && width(gap(g)) > 0 )
+	    { int tmp;
+
+	      FixAndPrintObject(prev, mk, back(prev, dim), fwd(prev, dim) + inc,
+	        dim, NO_SUPPRESS, pg, count);
+
+	      gaps_sofar++;
+	      tmp = ((frame_size - actual_size) * gaps_sofar) / adjustable_gaps;
+	      mk += save_actual_gap(g) + (tmp - adjust_sofar);
+	      adjust_sofar = tmp;
+	    }
+	    else
+	    {
+	      FixAndPrintObject(prev, mk, back(prev, dim), fwd(prev, dim),
+	        dim, NO_SUPPRESS, pg, count);
+
+	      mk += save_actual_gap(g);
+	    }
+	    prev = y;
+
+	    /* commence adjustment if required */
+	    if( !adjusting && will_adjust && g == last_bad_gap )
+	      adjusting = TRUE;
+
+	    NextDefiniteWithGap(x, link, y, g, jn);
+	  }
 
-	underlining = FALSE;
-	adjusting = will_adjust && last_bad_gap == nilobj;
-	FirstDefinite(x, link, y, jn);
-	prev = y;
-	mk = xmk - back(x, dim) + back(y, dim) + adjust_indent;
-	NextDefiniteWithGap(x, link, y, g, jn);
-	while( link != x )
-	{
 	  /* check for underlining */
-	  if( underline(prev) )
+	  debugcond3(DGP, D, underline(prev) == UNDER_UNDEF,
+	    "  underlining is UNDER_UNDEF in %s: %s %s in para:",
+	    EchoFilePos(&fpos(prev)), Image(type(prev)), EchoObject(prev));
+	  debugcond1(DGP, D, underline(prev)==UNDER_UNDEF, "%s", EchoObject(x));
+	  assert( underline(prev) == UNDER_OFF || underline(prev) == UNDER_ON,
+	    "FixAndPrint: underline(prev)!" );
+	  if( underline(prev) == UNDER_ON )
 	  {
 	    debug3(DGP, DD, "  FAPO/ACAT1 underline() := %s for %s %s",
 	      bool(FALSE), Image(type(prev)), EchoObject(prev));
-	    underline(prev) = FALSE;
+	    underline(prev) = UNDER_OFF;
 	    if( !underlining )
 	    {
 	      /* underlining begins here */
-	      underlining = TRUE;
 	      debug2(DGP, DD, "underlining begins at %s %s",
-		Image(type(prev)), EchoObject(prev));
+	        Image(type(prev)), EchoObject(prev));
+	      underlining = TRUE;
 	      underline_font = is_word(type(prev)) ? word_font(prev) :
-		  font(save_style(x));
+		    font(save_style(x));
 	      underline_xstart = mk - back(prev, dim);
 	    }
-	    if( !underline(g) )
-	    {
-	      /* underlining ends here */
-	      debug2(DGP, DD, "underlining ends at %s %s",
-		Image(type(prev)), EchoObject(prev));
-	      New(urec, UNDER_REC);
-	      back(urec, COLM) = underline_xstart;
-	      fwd(urec, COLM) = mk + fwd(prev, dim);
-	      back(urec, ROWM) = underline_font;
-	      underlining = FALSE;
-	      Link(Up(prev), urec);
-	    }
-	  }
 
-	  /* fix previous definite now we know it is not the last one  */
-	  if( adjusting && width(gap(g)) > 0 )
-	  { int tmp;
-
-	    FixAndPrintObject(prev, mk, back(prev, dim), fwd(prev, dim) + inc,
-	      dim, NO_SUPPRESS, pg, count);
-
-	    gaps_sofar++;
-	    tmp = ((frame_size - actual_size) * gaps_sofar) / adjustable_gaps;
-	    mk += save_actual_gap(g) + (tmp - adjust_sofar);
-	    adjust_sofar = tmp;
-	  }
-	  else
-	  {
-	    FixAndPrintObject(prev, mk, back(prev, dim), fwd(prev, dim),
-	      dim, NO_SUPPRESS, pg, count);
-
-	    mk += save_actual_gap(g);
-	  }
-	  prev = y;
-
-	  /* commence adjustment if required */
-	  if( !adjusting && will_adjust && g == last_bad_gap )
-	    adjusting = TRUE;
-
-	  NextDefiniteWithGap(x, link, y, g, jn);
-	}
-
-	/* check for underlining */
-	debug3(DGP, DD, "  underlining is %s in %s %s", bool(underline(prev)),
-	  Image(type(prev)), EchoObject(prev));
-	if( underline(prev) )
-	{
-	  debug3(DGP, DD, "  FAPO/ACAT1 underline() := %s for %s %s",
-	    bool(FALSE), Image(type(prev)), EchoObject(prev));
-	  underline(prev) = FALSE;
-	  if( !underlining )
-	  {
-	    /* underlining begins here */
-	    debug2(DGP, DD, "underlining begins at %s %s",
+	    /* underlining must end here */
+	    debug2(DGP, DD, "underlining ends at %s %s",
 	      Image(type(prev)), EchoObject(prev));
-	    underlining = TRUE;
-	    underline_font = is_word(type(prev)) ? word_font(prev) :
-		  font(save_style(x));
-	    underline_xstart = mk - back(prev, dim);
+	    New(urec, UNDER_REC);
+	    back(urec, COLM) = underline_xstart;
+	    fwd(urec, COLM) = mk + fwd(prev, dim);
+	    back(urec, ROWM) = underline_font;
+	    underlining = FALSE;
+	    Link(Up(prev), urec);
 	  }
 
-	  /* underlining must end here */
-	  debug2(DGP, DD, "underlining ends at %s %s",
-	    Image(type(prev)), EchoObject(prev));
-	  New(urec, UNDER_REC);
-	  back(urec, COLM) = underline_xstart;
-	  fwd(urec, COLM) = mk + fwd(prev, dim);
-	  back(urec, ROWM) = underline_font;
-	  underlining = FALSE;
-	  Link(Up(prev), urec);
-	}
-
-	/* fix the last definite subobject, prev, which must exist */
-	FixAndPrintObject(prev, mk, back(prev, dim),
-	  frame_size - (mk - xmk) - back(x, dim),
-	  dim, NO_SUPPRESS, pg, count);
+	  /* fix the last definite subobject, prev, which must exist */
+	  FixAndPrintObject(prev, mk, back(prev, dim),
+	    frame_size - (mk - xmk) - back(x, dim),
+	    dim, NO_SUPPRESS, pg, count);
 
+        }
       }
       else for( link = Down(x);  link != x;  link = NextDown(link) )
       {	Child(y, link);
@@ -946,10 +1201,19 @@ void FixAndPrintObject(OBJECT x, FULL_LENGTH xmk, FULL_LENGTH xb,
 	link = NextDown(link), uplink = NextUp(uplink), i++ );
       assert( link != x && uplink != x, "FixAndPrintObject: link or uplink!" );
       CountChild(y, link, count);
+      debug7(DGP, DD, "  fapo of %s (%s,%s) child %d %s (%s,%s)",
+	Image(type(x)),
+	EchoLength(back(x, dim)), EchoLength(fwd(x, dim)),
+	i, Image(type(y)), EchoLength(back(y, dim)), EchoLength(fwd(y, dim)));
       MoveLink(uplink, link, CHILD);  DeleteLink(link);  /* IMPORTANT!!! */
       assert( type(y) != GAP_OBJ, "FAPO: THR!");
 
       /* assign size if not done previously */
+      /* ***
+      if( thr_state(x) != FINALSIZE && type(y) != START_HVSPAN &&
+	  type(y) != START_HSPAN && type(y) != START_VSPAN &&
+	  type(y) != HSPAN && type(y) != VSPAN )
+      *** */
       if( thr_state(x) != FINALSIZE )
       {	back(x, dim) = xb;  fwd(x, dim) = xf;
 	thr_state(x) = FINALSIZE;
