@@ -1,16 +1,18 @@
 /*@z24.c:Print Service:PrintInit()@*******************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
 /*  The University of Sydney 2006                                            */
 /*  AUSTRALIA                                                                */
 /*                                                                           */
+/*  PDF Back End by Vincent Tan, February 1998.                              */
+/*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,7 +22,7 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z24.c                                                      */
 /*  MODULE:       Print Service                                              */
@@ -33,7 +35,9 @@
 /*  This module implements the PostScript back end.                          */
 /*                                                                           */
 /*****************************************************************************/
-#include "externs"
+#include <math.h>			/* for fabs()                        */
+
+#include "externs.h"
 #define	StartUpResource	"LoutStartUp"
 #define DEFAULT_XHEIGHT 500
 #define	NO_FONT		0		/* actually stolen from z37.c        */
@@ -389,6 +393,21 @@ void PrintBeforeFirst(FULL_LENGTH h, FULL_LENGTH v, FULL_CHAR *label)
       fprintf(out_fp, "%%%%EndPageSetup\n\n");
       break;
 
+    case PDF:
+
+      PDFFile_Init(out_fp, h/PT, v/PT, IN, CM, PT, EM);
+
+      /* print encoding vectors as resources */
+	MapPrintEncodings(out_fp);
+
+	FontPrintPageSetup(out_fp);
+
+      PDFPage_Init(out_fp, 1.0 / PT, PT/2);
+
+    	FontPrintPageResources(out_fp);	/*	write out font objects	*/
+      FontAdvanceCurrentPage();
+      break;
+
  } /* end switch */
  prologue_done = TRUE;
 } /* end PrintBeforeFirst */
@@ -486,6 +505,18 @@ void PrintBetween(FULL_LENGTH h, FULL_LENGTH v, FULL_CHAR *label)
       wordcount = 0;
       break;
 
+    case PDF:
+
+      /* write out page objects	*/
+      PDFPage_Cleanup(out_fp);
+      PDFPage_Init(out_fp, 1.0 / PT, PT/2);
+
+      /* write out font objects	*/
+      FontPrintPageResources(out_fp);
+      FontPrintPageSetup(out_fp);
+      FontAdvanceCurrentPage();
+      break;
+
   } /* end switch */
 } /* end PrintBetween */
 
@@ -527,20 +558,30 @@ void PrintWord(OBJECT x, int hpos, int vpos)
 
   debug5(DGP, DD, "PrintWord( %s, %d, %d ) font %d colour %d", string(x),
 	hpos, vpos, word_font(x), word_colour(x));
+  TotalWordCount++;
 
   switch( BackEnd )
   {
     case PLAINTEXT:
 
       h = ((float) hpos / PlainCharWidth) + 0.5;
-      /*** buggy; mark is already halfway up the word ***
-      v = ((float) vpos / PlainCharHeight) + 0.5;
-      *** */
       v = ((float) vpos / PlainCharHeight);
-      debug3(DGP, D, "PrintWord(%s at h = %d, v = %d)", string(x), h, v);
-      p = &page[v*hsize + h];
-      for( i = 0;  string(x)[i] != '\0';  i++ )
-	*p++ = string(x)[i];
+      debug3(DGP, DD, "PrintWord(%s at h = %d, v = %d)", string(x), h, v);
+      if( h >= 0 && h + StringLength(string(x)) < hsize && v >= 0 && v < vsize )
+      {
+        assert( h >= 0,     "PrintWord:  h < 0!" );
+        assert( h <  hsize, "PrintWord:  h >= hsize!" );
+        assert( v >= 0,     "PrintWord:  v < 0!" );
+        assert( v <  vsize, "PrintWord:  v >= vsize!" );
+        p = &page[v*hsize + h];
+        for( i = 0;  string(x)[i] != '\0';  i++ )
+	  *p++ = string(x)[i];
+      }
+      else
+      {
+        Error(24, 11, "word %s deleted (internal error, off page at %d,%d)",
+          WARN, &fpos(x), string(x), h, v);
+      }
       break;
 
 
@@ -640,11 +681,213 @@ void PrintWord(OBJECT x, int hpos, int vpos)
       else fprintf(out_fp, ")%c ", command);
       break;
 
+
+
+    case PDF:
+    {
+
+      static int last_hpos;	/* does not need to be initialised */
+      static int next_hpos = -1;
+#if 0
+      struct metrics *fnt;
+#endif
+      /* if font is different to previous word then print change */
+      if( word_font(x) != currentfont )
+      { currentfont = word_font(x);
+        currentxheight2 = FontHalfXHeight(currentfont);
+	  PDFFont_Set(out_fp, FontSize(currentfont, x), FontName(currentfont));
+      }
+
+      /* if colour is different to previous word then print change */
+      if( word_colour(x) != currentcolour )
+      {
+        currentcolour = word_colour(x);
+	  if( currentcolour > 0 )
+	  {
+          char	str[256];
+
+	    sprintf(str, "%s ", ColourCommand(currentcolour));
+	    PDFPage_Write(out_fp, str);
+        }
+      }
+
+      /* move to coordinate of x */
+      debug1(DGP, DDD, "  currentxheight2 = %d", currentxheight2);
+      vpos = vpos - currentxheight2;
+      if( cpexists && (currenty == vpos) && PDFHasValidTextMatrix() )
+      { /* printnum(hpos, out_fp); */
+        command = 's';
+/*
+	Note: I calculate the width of the space char here in case the font has
+	changed. This prevents subtle spacing errors from occurring.
+ */
+#if 0
+        fnt = finfo[currentfont].size_table;
+
+        if ( (next_hpos + fnt[' '].right /* width of space char */ ) == hpos )
+          command = ' ';
+#endif
+      }
+      else
+      { currenty = vpos;
+        /* printnum(hpos, out_fp);
+        fputs(" ", out_fp);
+        printnum(currenty, out_fp); */
+        command = 'm';
+        cpexists = TRUE;
+      }
+
+      /* convert ligature sequences into ligature characters */
+      lig = finfo[word_font(x)].lig_table;
+      p = q = string(x);
+      do
+      { 
+        /* check for missing glyph (lig[] == 1) or ligatures (lig[] > 1) */
+        if( lig[*q++ = *p++] )
+        {
+          if( lig[*(q-1)] == 1 ) continue;
+          else
+          { a = &lig[ lig[*(p-1)] + MAX_CHARS ];
+            while( *a++ == *(p-1) )
+            { b = p;
+              while( *a == *b && *(a+1) != '\0' && *b != '\0' )  a++, b++;
+              if( *(a+1) == '\0' )
+              { *(q-1) = *a;
+                p = b;
+                break;
+              }
+              else
+              { while( *++a );
+                a++;
+              }
+            }
+          }
+        }
+      } while( *p );
+      *q = '\0';
+
+      /* show string(x) */
+      /* FontWordSize(x); - this should not be necessary	*/
+
+      switch (command)
+      {
+	case 'm':
+
+	  PDFText_OpenXY(out_fp, hpos, vpos);
+	  last_hpos = hpos;
+	  next_hpos =	hpos + fwd(x, COLM);	/* fwd(x, COLM) = width of wd */
+	  break;
+
+	case 's':
+#if 0
+	  PDFText_Open(out_fp);
+	  PDFText_Kern(out_fp, hpos - next_hpos);
+#else
+	  PDFText_OpenX(out_fp, hpos - last_hpos);
+#endif
+	  last_hpos = hpos;
+	  next_hpos =	hpos + fwd(x, COLM);	/* fwd(x, COLM) = width of wd */
+	  break;
+#if 0
+	case ' ':
+
+	  PDFText_Open(out_fp);
+#if 1
+	  /* try kerning to get correct position */
+	  PDFText_Kern(out_fp, fnt[' '].right);
+#else
+	  PDFPage_Write(out_fp, EightBitToPrintForm[' ']);
+#endif
+	  next_hpos += fwd(x, COLM) +	fnt[' '].right;	/* width of space ch */
+	  break;
+#endif
+      }
+
+      p = string(x);
+      PDFPage_Write(out_fp, EightBitToPrintForm[*p]);
+
+      m = font_mapping(finfo[word_font(x)].font_table);
+      unacc = MapTable[m]->map[MAP_UNACCENTED];
+      /* acc   = MapTable[m]->map[MAP_ACCENT]; */
+      for( p++;  *p;  p++ )
+      {
+	KernLength(word_font(x), unacc, *(p-1), *p, ksize);
+        KernLength(font_num(finfo[word_font(x)].original_font),
+	  unacc, *(p-1), *p, ksize);
+	if ( ksize != 0 )
+	{
+	  PDFText_Kern(out_fp, ksize);
+	}
+	PDFPage_Write(out_fp, EightBitToPrintForm[*p]);
+      }
+      PDFText_Close(out_fp);
+      break;
+    }
+
+
   } /* end switch */
   debug0(DGP, DDD, "PrintWord returning");
 } /* end PrintWord */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  PrintPlainGraphicObject(x, xmk, ymk, z)                                  */
+/*                                                                           */
+/*  Print plain graphic object x at xmk, ymk with the size of z.             */
+/*                                                                           */
+/*****************************************************************************/
+
+void PrintPlainGraphicObject(OBJECT x, FULL_LENGTH xmk,FULL_LENGTH ymk,OBJECT z)
+{ int i, len, starth, startv, stoph, stopv, h, v;
+  debug2(DGP, D, "PrintPlainGraphicObject(x, xmk %s, ymk %s)",
+    EchoLength(xmk), EchoLength(ymk));
+
+  assert( BackEnd == PLAINTEXT, "PrintPlainGraphicObject: back end!" );
+  if( type(x) != WORD && type(x) != QWORD )
+  {
+    Error(24, 12, "left parameter of %s must be a simple word",
+      WARN, &fpos(x), KW_PLAINGRAPHIC);
+    return;
+  }
+  len = StringLength(string(x));
+  if( StringLength(string(x)) == 0 )
+  {
+    Error(24, 13, "left parameter of %s must be a non-empty word",
+      WARN, &fpos(x), KW_PLAINGRAPHIC);
+    return;
+  }
+  starth = (((float) xmk ) / PlainCharWidth) + 0.5;
+  startv = (((float) ymk ) / PlainCharHeight);
+  stoph = (((float) xmk + size(z, COLM)) / PlainCharWidth) + 0.5;
+  stopv = (((float) ymk - size(z, ROWM)) / PlainCharHeight); /* NB - not + */
+  SetLengthDim(COLM);
+  debug5(DGP, D, "  xmk %s bk %s fwd %s -> %d,%d",
+    EchoLength(xmk), EchoLength(back(z, COLM)), EchoLength(fwd(z, COLM)),
+    starth, stoph);
+  SetLengthDim(ROWM);
+  debug5(DGP, D, "  ymk %s bk %s fwd %s -> %d,%d",
+    EchoLength(ymk), EchoLength(back(z, ROWM)), EchoLength(fwd(z, ROWM)),
+    startv, stopv);
+  if( starth >= 0 && stoph < hsize && startv >= 0 && stopv < vsize )
+  { i = 0;
+    for( v = startv-1;  v >= stopv;  v-- )
+    {
+      for( h = starth;  h < stoph;  h++ )
+      {
+        if( i == len )  i = 0;
+        page[v*hsize + h] = string(x)[i++];
+      }
+    }
+  }
+  else
+  {
+    Error(24, 14, "fill %s deleted (internal error, off page at %d,%d)",
+      WARN, &fpos(x), string(x), h, v);
+  }
+} /* end PrintPlainGraphicObject */
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  PrintUnderline(fnum, xstart, xstop, ymk)                                 */
@@ -675,6 +918,10 @@ void PrintUnderline(FONT_NUM fnum, FULL_LENGTH xstart, FULL_LENGTH xstop,
 	ymk - finfo[fnum].underline_pos, finfo[fnum].underline_thick);
       break;
 
+    case PDF:
+	PDFPage_PrintUnderline(out_fp,  xstart, xstop,
+	    ymk - finfo[fnum].underline_pos, finfo[fnum].underline_thick);
+      break;
   }
   debug0(DGP, DD, "PrintUnderline returning.");
 } /* end PrintUnderline */
@@ -751,6 +998,13 @@ void PrintAfterLast(void)
         fprintf(out_fp, "%%%%EOF\n");
 	break;
 
+     case PDF:
+
+    	PDFPage_Cleanup(out_fp);		/* write out page objects */
+        /* MapPrintResources(out_fp); not needed */
+    	PDFFile_Cleanup(out_fp);
+        break;
+
     } /* end switch */
   } /* end if prologue_done */
 } /* end PrintAfterLast */
@@ -767,8 +1021,22 @@ void PrintAfterLast(void)
 void CoordTranslate(FULL_LENGTH xdist, FULL_LENGTH ydist)
 { debug2(DRS,D,"CoordTranslate(%s, %s)",
     EchoLength(xdist), EchoLength(ydist));
-  assert( BackEnd == POSTSCRIPT, "CoordTranslate: BackEnd!" );
-  fprintf(out_fp, "%d %d translate\n", xdist, ydist);
+  assert( BackEnd != PLAINTEXT, "CoordTranslate: BackEnd!" );
+  if (BackEnd == POSTSCRIPT)
+    fprintf(out_fp, "%d %d translate\n", xdist, ydist);
+  else if (BackEnd == PDF)
+  {
+    if ((xdist != 0) || (ydist != 0))
+    {
+#if 1
+      PDFPage_Translate(out_fp, xdist, ydist);
+#else
+      char	temp_str[64];
+      sprintf(temp_str, "1 0 0 1 %d %d cm\n", xdist, ydist);
+      PDFPage_Write(out_fp, temp_str);
+#endif
+    }
+  }
   cpexists = FALSE;
   /***
   currentfont = NO_FONT;
@@ -777,6 +1045,7 @@ void CoordTranslate(FULL_LENGTH xdist, FULL_LENGTH ydist)
   debug0(DRS, D, "CoordTranslate returning.");
 } /* end CoordTranslate */
 
+
 /*@::CoordRotate(), CoordScale(), SaveGraphicsState(), etc.@******************/
 /*                                                                           */
 /*  CoordRotate(amount)                                                      */
@@ -787,8 +1056,19 @@ void CoordTranslate(FULL_LENGTH xdist, FULL_LENGTH ydist)
 
 void CoordRotate(FULL_LENGTH amount)
 { debug1(DRS, D, "CoordRotate(%.1f degrees)", (float) amount / DG);
-  assert( BackEnd == POSTSCRIPT, "CoordRotate: BackEnd!" );
-  fprintf(out_fp, "%.4f rotate\n", (float) amount / DG);
+  assert( BackEnd != PLAINTEXT, "CoordRotate: BackEnd!" );
+  if (BackEnd == POSTSCRIPT)
+    fprintf(out_fp, "%.4f rotate\n", (float) amount / DG);
+  else if (BackEnd == PDF)
+  {
+    int theAmount = ((amount / DG) % 360);
+    if ( theAmount != 0 )
+    {
+      #define  PI   3.1415926535897931160
+
+      PDFPage_Rotate(out_fp, (double) theAmount * (double) PI / (double) 180.0);
+    }
+  }
   cpexists = FALSE;
   /***
   currentfont = NO_FONT;
@@ -811,10 +1091,24 @@ void CoordScale(float hfactor, float vfactor)
 #if DEBUG_ON
   char buff[20];
 #endif
-  assert( BackEnd == POSTSCRIPT, "CoordScale: BackEnd!" );
+  assert( BackEnd != PLAINTEXT, "CoordScale: BackEnd!" );
   ifdebug(DRS, D, sprintf(buff, "%.3f, %.3f", hfactor, vfactor));
   debug1(DRS, D, "CoordScale(%s)", buff);
-  fprintf(out_fp, "%.4f %.4f scale\n", hfactor, vfactor);
+  if (BackEnd == POSTSCRIPT)
+    fprintf(out_fp, "%.4f %.4f scale\n", hfactor, vfactor);
+  else if (BackEnd == PDF)
+  {
+    if ( (fabs(hfactor - 1.0) > 0.01) || (fabs(vfactor - 1.0) > 0.01) )
+    {
+#if 1
+      PDFPage_Scale(out_fp, hfactor, vfactor);
+#else
+      char temp_str[64];
+      sprintf(temp_str, "%.2f 0 0 %.2f 0 0 cm\n", hfactor, vfactor);
+      PDFPage_Write(out_fp, temp_str);
+#endif
+    }
+  }
   cpexists = FALSE;
   /***
   currentfont = NO_FONT;
@@ -835,11 +1129,14 @@ void CoordScale(float hfactor, float vfactor)
 
 void SaveGraphicState(OBJECT x)
 { debug0(DRS, D, "SaveGraphicState()");
-  assert( BackEnd == POSTSCRIPT, "SaveGraphicState: BackEnd!" );
-  fprintf(out_fp, "gsave\n");
+  assert( BackEnd != PLAINTEXT, "SaveGraphicState: BackEnd!" );
+  if (BackEnd == POSTSCRIPT)
+    fprintf(out_fp, "gsave\n");
+  else if (BackEnd == PDF)
+    PDFPage_Push(out_fp);
   gs_stack_top++;
   if( gs_stack_top >= MAX_GS )
-    Error(24, 9, "rotations, graphics etc. too deeply nested (max is %d)",
+    Error(24, 5, "rotations, graphics etc. too deeply nested (max is %d)",
       FATAL, &fpos(x), MAX_GS);
   gs_stack[gs_stack_top].gs_font	= currentfont;
   gs_stack[gs_stack_top].gs_colour	= currentcolour;
@@ -863,8 +1160,11 @@ void SaveGraphicState(OBJECT x)
 
 void RestoreGraphicState(void)
 { debug0(DRS, D, "RestoreGraphicState()");
-  assert( BackEnd == POSTSCRIPT, "RestoreGraphicState: BackEnd!" );
-  fprintf(out_fp, "\ngrestore\n");
+  assert( BackEnd != PLAINTEXT, "RestoreGraphicState: BackEnd!" );
+  if( BackEnd == POSTSCRIPT )
+    fprintf(out_fp, "\ngrestore\n");
+  else if( BackEnd == PDF )
+    PDFPage_Pop(out_fp);
   currentfont	  = gs_stack[gs_stack_top].gs_font;
   currentcolour	  = gs_stack[gs_stack_top].gs_colour;
   cpexists	  = gs_stack[gs_stack_top].gs_cpexists;
@@ -890,13 +1190,23 @@ void RestoreGraphicState(void)
 
 void PrintGraphicObject(OBJECT x)
 { OBJECT y, link;
-  assert( BackEnd == POSTSCRIPT, "PrintGraphicObject: BackEnd!" );
+  assert( BackEnd != PLAINTEXT, "PrintGraphicObject: BackEnd!" );
+  debug3(DPS, D, "PrintGraphicObject(%s %s %s)",
+    EchoFilePos(&fpos(x)), Image(type(x)), EchoObject(x));
   switch( type(x) )
   {
     case WORD:
     case QWORD:
-    
+#if 1
+      if (BackEnd == POSTSCRIPT)
+	StringFPuts(string(x), out_fp);
+      else if (BackEnd == PDF)
+      {
+	PDFPage_WriteGraphic(out_fp, string(x));
+      }
+#else
       StringFPuts(string(x), out_fp);
+#endif
       break;
 	
 
@@ -905,13 +1215,23 @@ void PrintGraphicObject(OBJECT x)
       for( link = Down(x);  link != x;  link = NextDown(link) )
       {	Child(y, link);
 	if( type(y) == GAP_OBJ )
-	{ if( vspace(y) > 0 )  fputs("\n", out_fp);
-	  else if( hspace(y) > 0 ) fputs(" ", out_fp);
+	{
+          if( BackEnd == POSTSCRIPT )
+	  {
+	    if( vspace(y) > 0 )  fputs("\n", out_fp);
+	    else if( hspace(y) > 0 ) fputs(" ", out_fp);
+	  }
+	  else if( BackEnd == PDF )
+	  {
+	    if( vspace(y) > 0 )  PDFPage_Write(out_fp, "\n");
+	    else if( hspace(y) > 0 ) PDFPage_Write(out_fp, " ");
+	  }
 	}
-	else if( is_word(type(y)) || type(y) == ACAT )  PrintGraphicObject(y);
+	else if( is_word(type(y)) || type(y) == ACAT )
+	  PrintGraphicObject(y);
 	else if( type(y) != WIDE && !is_index(type(y)) )
 		/* @Wide, indexes are sometimes inserted by Manifest */
-	{ Error(24, 5, "error in left parameter of %s",
+	{ Error(24, 6, "error in left parameter of %s",
 	    WARN, &fpos(x), KW_GRAPHIC);
 	  debug1(DGP, D, "  type(y) = %s, y =", Image(type(y)));
 	  ifdebug(DGP, D, DebugObject(y));
@@ -922,11 +1242,13 @@ void PrintGraphicObject(OBJECT x)
 
     default:
     
-      Error(24, 6, "error in left parameter of %s", WARN, &fpos(x), KW_GRAPHIC);
+      Error(24, 7, "error in left parameter of %s", WARN, &fpos(x), KW_GRAPHIC);
       debug1(DGP, D, "  type(x) = %s, x =", Image(type(x)));
       ifdebug(DGP, D, DebugObject(x));
       break;
+
   }
+  debug0(DPS, D, "PrintGraphicObject returning");
 } /* end PrintGraphicObject */
 
 
@@ -940,7 +1262,7 @@ void PrintGraphicObject(OBJECT x)
 
 void DefineGraphicNames(OBJECT x)
 { assert( type(x) == GRAPHIC, "PrintGraphic: type(x) != GRAPHIC!" );
-  assert( BackEnd == POSTSCRIPT, "DefineGraphicNames: BackEnd!" );
+  assert( BackEnd != PLAINTEXT, "DefineGraphicNames: BackEnd!" );
   debug1(DRS, D, "DefineGraphicNames( %s )", EchoObject(x));
   debug1(DRS, DD, "  style = %s", EchoStyle(&save_style(x)));
 
@@ -949,8 +1271,15 @@ void DefineGraphicNames(OBJECT x)
   { currentfont = font(save_style(x));
     if( currentfont > 0 )
     { currentxheight2 = FontHalfXHeight(currentfont);
+#if 1	/*	VT 98/01/04: modified for PDF	*/
+      if (BackEnd == POSTSCRIPT)
+        fprintf(out_fp, "%hd %s ", FontSize(currentfont, x), FontName(currentfont));
+      else if (BackEnd == PDF)
+	PDFFont_Set(out_fp, FontSize(currentfont, x), FontName(currentfont));
+#else
       fprintf(out_fp, "%hd %s ", FontSize(currentfont, x),
         FontName(currentfont));
+#endif
     }
   }
 
@@ -958,14 +1287,33 @@ void DefineGraphicNames(OBJECT x)
   if( colour(save_style(x)) != currentcolour )
   { currentcolour = colour(save_style(x));
     if( currentcolour > 0 )
-    { fprintf(out_fp, "%s ", ColourCommand(currentcolour));
+    {
+#if 1
+      if( BackEnd == POSTSCRIPT )
+        fprintf(out_fp, "%s ", ColourCommand(currentcolour));
+      else if (BackEnd == PDF)
+      {
+	char str[256];
+	sprintf(str, "%s ", ColourCommand(currentcolour));
+	PDFPage_Write(out_fp, str);
+      }
+#else
+      fprintf(out_fp, "%s ", ColourCommand(currentcolour));
+#endif
     }
   }
 
-  fprintf(out_fp, "%d %d %d %d %d %d %d LoutGraphic\n",
-    size(x, COLM), size(x, ROWM), back(x, COLM), fwd(x, ROWM),
-    currentfont <= 0 ? 12*PT : FontSize(currentfont, x),
-    width(line_gap(save_style(x))), width(space_gap(save_style(x))));
+  if (BackEnd == POSTSCRIPT)
+    fprintf(out_fp, "%d %d %d %d %d %d %d LoutGraphic\n",
+      size(x, COLM), size(x, ROWM), back(x, COLM), fwd(x, ROWM),
+      currentfont <= 0 ? 12*PT : FontSize(currentfont, x),
+      width(line_gap(save_style(x))), width(space_gap(save_style(x))));
+  else if( BackEnd == PDF )
+  {
+    PDFPage_SetVars(size(x, COLM), size(x, ROWM), back(x, COLM), fwd(x, ROWM),
+      currentfont <= 0 ? 12*PT : FontSize(currentfont, x),
+      width(line_gap(save_style(x))), width(space_gap(save_style(x))));
+  }
 
   debug0(DRS, D, "DefineGraphicNames returning.");
 } /* end DefineGraphicNames */
@@ -988,6 +1336,17 @@ void DefineGraphicNames(OBJECT x)
 
 void SaveTranslateDefineSave(OBJECT x, FULL_LENGTH xdist, FULL_LENGTH ydist)
 {
+  if( BackEnd == PDF )
+  {
+    /* do it bit by bit */
+    SaveGraphicState(x);
+    CoordTranslate(xdist, ydist);
+    DefineGraphicNames(x);
+    SaveGraphicState(x);
+    return;
+  }
+
+  assert( BackEnd == POSTSCRIPT, "SaveTranslateDefineSave: BackEnd!" );
   if( gs_stack_top >= MAX_GS - 1 || font(save_style(x)) != currentfont ||
       colour(save_style(x))!=currentcolour )
   {
@@ -1030,6 +1389,7 @@ void SaveTranslateDefineSave(OBJECT x, FULL_LENGTH xdist, FULL_LENGTH ydist)
   }
 } /* end SaveTranslateDefineSave */
 
+
 /*@::PrintGraphicInclude()@***************************************************/
 /*                                                                           */
 /*  PrintGraphicInclude(x, colmark, rowmark)                                 */
@@ -1063,6 +1423,14 @@ void PrintGraphicInclude(OBJECT x, FULL_LENGTH colmark, FULL_LENGTH rowmark)
 { OBJECT y, full_name;  FULL_CHAR buff[MAX_BUFF];
   FILE *fp;  int state;  BOOLEAN compressed;
   debug0(DRS, D, "PrintGraphicInclude(x)");
+
+  if (BackEnd == PDF)
+  {
+    Error(24, 8, "PrintGraphicInclude: cannot include EPSF in a PDF file. File ignored.",
+      WARN, &fpos(x));
+    return;
+  }
+
   assert( BackEnd == POSTSCRIPT, "PrintGraphicInclude: BackEnd!" );
   assert(type(x)==INCGRAPHIC || type(x)==SINCGRAPHIC, "PrintGraphicInclude!");
   assert(incgraphic_ok(x), "PrintGraphicInclude: !incgraphic_ok(x)!");
@@ -1076,14 +1444,15 @@ void PrintGraphicInclude(OBJECT x, FULL_LENGTH colmark, FULL_LENGTH rowmark)
   if( font(save_style(x)) != currentfont )
   { currentfont = font(save_style(x));
     currentxheight2 = FontHalfXHeight(currentfont);
-    fprintf(out_fp, "%hd %s\n", FontSize(currentfont, x), FontName(currentfont));
+    fprintf(out_fp, "%hd %s\n", FontSize(currentfont,x), FontName(currentfont));
   }
 
   /* if colour is different to previous word then print change */
   if( colour(save_style(x)) != currentcolour )
   { currentcolour = colour(save_style(x));
     if( currentcolour > 0 )
-    { fprintf(out_fp, "%s\n", ColourCommand(currentcolour));
+    {
+      fprintf(out_fp, "%s\n", ColourCommand(currentcolour));
     }
   }
 
@@ -1110,10 +1479,10 @@ void PrintGraphicInclude(OBJECT x, FULL_LENGTH colmark, FULL_LENGTH rowmark)
       }
       else
       { if( StringBeginsWith(buff, AsciiToFull("%%LanguageLevel:")) )
-	  Error(24, 7, "ignoring LanguageLevel comment in %s file %s",
+	  Error(24, 9, "ignoring LanguageLevel comment in %s file %s",
 	    WARN, &fpos(x), KW_INCGRAPHIC, string(full_name));
 	if( StringBeginsWith(buff, AsciiToFull("%%Extensions:")) )
-	  Error(24, 8, "ignoring Extensions comment in %s file %s",
+	  Error(24, 10, "ignoring Extensions comment in %s file %s",
 	    WARN, &fpos(x), KW_INCGRAPHIC, string(full_name));
 	if( !strip_out(buff) )  StringFPuts(buff, out_fp);
 	state = (StringFGets(buff, MAX_BUFF, fp) == NULL) ? FINISHED : SKIPPING;
@@ -1138,7 +1507,7 @@ void PrintGraphicInclude(OBJECT x, FULL_LENGTH colmark, FULL_LENGTH rowmark)
   DisposeObject(full_name);
   fclose(fp);
   if( compressed )  StringRemove(AsciiToFull(LOUT_EPS));
-  fprintf(out_fp, "%%%%EndDocument\nEndEPSF\n");
+  fprintf(out_fp, "\n%%%%EndDocument\nEndEPSF\n");
   wordcount = 0;
   debug0(DRS, D, "PrintGraphicInclude returning.");
 } /* end PrintGraphicInclude */
