@@ -1,7 +1,7 @@
 /*@z32.c:Counter Service:Next()@**********************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.12)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -63,6 +63,7 @@ OBJECT Next(OBJECT x, int inc, BOOLEAN *done)
       word_colour(y) = word_colour(x);
       word_language(y) = word_language(x);
       word_hyph(y) = word_hyph(x);
+      underline(y) = underline(x);
       MergeNode(y, x);  x = y;
       *done = TRUE;
       break;
@@ -76,6 +77,11 @@ OBJECT Next(OBJECT x, int inc, BOOLEAN *done)
     case PAGE_LABEL:
     case CROSS:
     case FORCE_CROSS:
+    case START_HVSPAN:
+    case START_HSPAN:
+    case START_VSPAN:
+    case HSPAN:
+    case VSPAN:
     
       break;
 
@@ -88,6 +94,8 @@ OBJECT Next(OBJECT x, int inc, BOOLEAN *done)
     case VSHIFT:
     case HCONTRACT:
     case VCONTRACT:
+    case HLIMITED:
+    case VLIMITED:
     case HEXPAND:
     case VEXPAND:
     case PADJUST:
@@ -98,9 +106,11 @@ OBJECT Next(OBJECT x, int inc, BOOLEAN *done)
     case HCOVER:
     case VCOVER:
     case ROTATE:
+    case BACKGROUND:
     case SCALE:
     case KERN_SHRINK:
     case SPLIT:
+    case PLAIN_GRAPHIC:
     case GRAPHIC:
     
       Child(y, LastDown(x));
@@ -110,6 +120,7 @@ OBJECT Next(OBJECT x, int inc, BOOLEAN *done)
 
     case ACAT:
     
+      /* *** seems identical!
       link = LastDown(x);
       while( link != x && !*done )
       {	Child(y, link);
@@ -118,6 +129,7 @@ OBJECT Next(OBJECT x, int inc, BOOLEAN *done)
 	if( !*done )  link = PrevDown(link);
       }
       break;
+      *** */
 
 
     case COL_THR:
