@@ -1,7 +1,7 @@
 /*@z33.c:Database Service:OldCrossDb(), NewCrossDb(), SymToNum()@*************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -10,7 +10,7 @@
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,7 +20,7 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z33.c                                                      */
 /*  MODULE:       Database Service                                           */
@@ -30,7 +30,7 @@
 /*                                                                           */
 /*****************************************************************************/
 #define INIT_DBCHECK_NUM	107
-#include "externs"
+#include "externs.h"
 
 
 /*****************************************************************************/
@@ -232,10 +232,10 @@ OBJECT OldCrossDb, NewCrossDb;
 
 OBJECT DbCreate(OBJECT x)
 { OBJECT db = x;
-  debug1(DBS, D, "DbCreate(%s)", string(db));
+  debug1(DBS, DD, "DbCreate(%s)", string(db));
   assert( is_word(type(x)), "DbCreate: !is_word(type(x))" );
-  reading(db) = FALSE;  filep(db) = null;
-  debug1(DBS, D, "DbCreate returning %s", EchoObject(db));
+  reading(db) = FALSE;  db_filep(db) = null;
+  debug1(DBS, DD, "DbCreate returning %s", EchoObject(db));
   return db;
 } /* end DbCreate */
 
@@ -281,18 +281,6 @@ BOOLEAN check)
 	dfnum == NO_FILE ? AsciiToFull(".") : FileName(dfnum));
   assert(!reading(db), "DbInsert: insert into reading database");
 
-  /* open database index file if not already done */
-  if( filep(db) == null )
-  { if( StringLength(string(db)) + StringLength(NEW_INDEX_SUFFIX) >= MAX_BUFF )
-      Error(33, 2, "database file name %s%s is too long",
-	FATAL, no_fpos, string(db), NEW_INDEX_SUFFIX);
-    StringCopy(buff, string(db));
-    StringCat(buff, NEW_INDEX_SUFFIX);
-    filep(db) = StringFOpen(buff, WRITE_BINARY);
-    if( filep(db) == null )
-      Error(33, 3, "cannot write to database file %s", FATAL, &fpos(db), buff);
-  }
-
   /* if required, check that (sym, tag) not already inserted */
   if( check )
   { 
@@ -317,6 +305,18 @@ BOOLEAN check)
     }
   }
 
+  /* open database index file if not already done */
+  if( db_filep(db) == null )
+  { if( StringLength(string(db)) + StringLength(NEW_INDEX_SUFFIX) >= MAX_BUFF )
+      Error(33, 2, "database file name %s%s is too long",
+	FATAL, no_fpos, string(db), NEW_INDEX_SUFFIX);
+    StringCopy(buff, string(db));
+    StringCat(buff, NEW_INDEX_SUFFIX);
+    db_filep(db) = StringFOpen(buff, WRITE_BINARY);
+    if( db_filep(db) == null )
+      Error(33, 3, "cannot write to database file %s", FATAL, &fpos(db), buff);
+  }
+
   /* work out database index file entry and append it to file */
   if( dfnum != NO_FILE )
   { StringCopy(buff, FileName(dfnum));
@@ -324,13 +324,15 @@ BOOLEAN check)
   }
   else StringCopy(buff, AsciiToFull("."));
   SymToNum(db, sym, symnum, gall);
-  ifdebug(DBS, D,
+  ifdebug(DBS, DD,
   fprintf(stderr, "  -> %s%d&%s\t%s\t%ld\t%d\t%s\n", gall ? "0" : "", symnum,
     tag, seq, dfpos, dlnum, buff);
   );
-  fprintf(filep(db), "%s%d&%s\t%s\t%s\t%ld\t%d\t%s\n", gall ? "0" : "", symnum,
-    tag, seq, StringFiveInt(++extra_seq), dfpos, dlnum, buff);
-  debug0(DBS, D, "DbInsert returning.");
+  fprintf(db_filep(db), "%s%d&%s\t%s\t%s\t%ld\t%d\t%s\n", gall ? "0" : "",
+    symnum, tag, seq, StringFiveInt(++extra_seq), dfpos, dlnum, buff);
+
+  /* return */
+  debug0(DBS, DD, "DbInsert returning.");
   ifdebug(DPP, D, ProfileOff("DbInsert"));
 } /* end DbInsert */
 
@@ -348,31 +350,32 @@ void DbConvert(OBJECT db, BOOLEAN full_name)
 { FULL_CHAR oldname[MAX_BUFF+10], newname[MAX_BUFF];
   OBJECT link, y;
   ifdebug(DPP, D, ProfileOn("DbConvert"));
-  debug2(DBS, D, "DbConvert( %ld %s )", (long) db, string(db));
+  debug2(DBS, DD, "DbConvert( %ld %s )", (long) db, string(db));
   assert( !reading(db), "DbConvert: reading database");
   StringCopy(newname, string(db));
   StringCat(newname, INDEX_SUFFIX);
   StringCopy(oldname, string(db));
   StringCat(oldname, NEW_INDEX_SUFFIX);
-  if( filep(db) != null )
-  { for( link = Down(db);  link != db;  link = NextDown(link) )
+  if( db_filep(db) != null )
+  {
+    fprintf(db_filep(db), "00 %s %s\n", LOUT_VERSION, "database index file");
+    for( link = Down(db);  link != db;  link = NextDown(link) )
     { Child(y, link);
       assert( type(y) == CROSS_SYM || type(y) == ACAT, "DbConvert: y!" );
       if( type(y) != CROSS_SYM )  continue;
-      fprintf(filep(db), "%s %d %s\n",
+      fprintf(db_filep(db), "%s %d %s\n",
 	db_targ(link) ? "00target" : "00symbol",
 	number(link),
 	full_name ? FullSymName(symb(y), AsciiToFull(" ")) : SymName(symb(y)));
     }
-    fprintf(filep(db), "00 %s %s\n", LOUT_VERSION, "database index file");
-    fclose(filep(db));
-    debug2(DBS, D, "  calling SortFile(%s, %s)", oldname, newname);
-    SortFile( (char *) oldname, (char *) newname);
+    fclose(db_filep(db));
+    debug2(DBS, DD, "  calling SortFile(%s, %s)", oldname, newname);
+    SortFile(oldname, newname);
   }
   else StringRemove(newname);
   StringRemove(oldname);
   DeleteNode(db);
-  debug0(DBS, D, "DbConvert returning.");
+  debug0(DBS, DD, "DbConvert returning.");
   ifdebug(DPP, D, ProfileOff("DbConvert"));
 } /* end DbConvert */
 
@@ -386,16 +389,16 @@ void DbConvert(OBJECT db, BOOLEAN full_name)
 /*****************************************************************************/
 
 void DbClose(OBJECT db)
-{ if( db != nilobj && filep(db) != NULL )
-  {  fclose(filep(db));
-     filep(db) = NULL;
+{ if( db != nilobj && !in_memory(db) && db_filep(db) != NULL )
+  {  fclose(db_filep(db));
+     db_filep(db) = NULL;
   }
 } /* end DbClose */
 
 
 /*@::DbLoad()@****************************************************************/
 /*                                                                           */
-/*  OBJECT DbLoad(stem, fpath, create, symbs)                                */
+/*  OBJECT DbLoad(stem, fpath, create, symbs, in_mem)                        */
 /*                                                                           */
 /*  Open for reading the database whose index file name is string(stem).li.  */
 /*  This file has not yet been defined; its search path is fpath.  If it     */
@@ -404,14 +407,19 @@ void DbClose(OBJECT db)
 /*  symbs is an ACAT of CLOSUREs showing the symbols that the database may   */
 /*  contain; or nilobj if the database may contain any symbol.               */
 /*                                                                           */
+/*  If in_mem is true, this database index is to be kept in internal memory, */
+/*  rather than an external file, as a speed optimization.                   */
+/*                                                                           */
 /*****************************************************************************/
 
-OBJECT DbLoad(OBJECT stem, int fpath, BOOLEAN create, OBJECT symbs)
+OBJECT DbLoad(OBJECT stem, int fpath, BOOLEAN create, OBJECT symbs,
+  BOOLEAN in_mem)
 { FILE *fp;  OBJECT db, t, res, tag, par, sym, link, y;
-  int i, lnum, dlnum, num, count;  FILE_NUM index_fnum, dfnum;  long dfpos;
-  BOOLEAN gall;  FULL_CHAR line[MAX_BUFF], sym_name[MAX_BUFF];
+  int i, lnum, dlnum, num, count, leftp;
+  FILE_NUM index_fnum, dfnum;  long dfpos;
+  BOOLEAN gall;  FULL_CHAR line[MAX_BUFF], sym_name[MAX_BUFF]; char *gotline;
   ifdebug(DPP, D, ProfileOn("DbLoad"));
-  debug3(DBS, D, "[ DbLoad(%s, %d, %s, -)", string(stem), fpath, bool(create));
+  debug3(DBS, DD, "[ DbLoad(%s, %d, %s, -)", string(stem), fpath, bool(create));
 
   /* open or else create index file fp */
   debug0(DFS, D, "  calling DefineFile from DbLoad (1)");
@@ -505,24 +513,28 @@ OBJECT DbLoad(OBJECT stem, int fpath, BOOLEAN create, OBJECT symbs)
   assert( i > 0, "DbLoad: FileName(index_fnum)!" );
   StringCopy(&line[i], STR_EMPTY);
   db = MakeWord(WORD, line, &fpos(stem));
-  reading(db) = TRUE;  filep(db) = fp;
+  reading(db) = TRUE;
+  in_memory(db) = in_mem;
   if( symbs != nilobj )
   { assert( type(symbs) == ACAT, "DbLoad: type(symbs)!" );
     Link(db, symbs);
   }
   if( fp == null )
-  { debug1(DBS, D, "] DbLoad returning (empty) %s", string(db));
+  { debug1(DBS, DD, "] DbLoad returning (empty) %s", string(db));
+    db_filep(db) = null;
+    db_lines(db) = (LINE *) NULL;
     ifdebug(DPP, D, ProfileOff("DbLoad"));
     return db;
   }
 
   /* read header lines of index file, find its symbols */
-  left_pos(db) = 0;  lnum = 1;
-  while( StringFGets(line, MAX_BUFF, fp) != NULL )
+  leftp = 0;  lnum = 1;
+  gotline = StringFGets(line, MAX_BUFF, fp);
+  while( gotline != NULL )
   {
     if( line[0] != '0' || line[1] != '0' )  break;
     lnum++;
-    left_pos(db) = (int) ftell(fp);
+    leftp = (int) ftell(fp);
     gall = StringBeginsWith(line, AsciiToFull("00target "));
     sscanf( (char *) line, gall ? "00target %d" : "00symbol %d", &num);
     for( i = 9;  line[i] != CH_SPACE && line[i] != '\0';  i++ );
@@ -562,13 +574,35 @@ OBJECT DbLoad(OBJECT stem, int fpath, BOOLEAN create, OBJECT symbs)
     else
     { Error(33, 13, "undefined symbol in database file %s (line %d)",
 	WARN, &fpos(db), FileName(index_fnum), lnum);
-      debug1(DBS, D, "] DbLoad returning %s (error)", string(db));
-      fclose(filep(db));  filep(db) = null;  /* effectively an empty database */
+      debug1(DBS, DD, "] DbLoad returning %s (error)", string(db));
+      fclose(fp);
+      in_memory(db) = FALSE;
+      db_filep(db) = null;  /* subsequently treated like an empty database */
       ifdebug(DPP, D, ProfileOff("DbLoad"));
       return db;
     }
+    gotline = StringFGets(line, MAX_BUFF, fp);
   }
-  debug1(DBS, D, "] DbLoad returning %s", string(db));
+
+  /* if in_memory, go on to read the entire database index into memory */
+  if( in_memory(db) )
+  { int len;
+    if( gotline == NULL )
+      db_lines(db) = 0;
+    else
+    {
+      db_lines(db) = ReadLines(fp, FileName(index_fnum), line, &len);
+      db_lineslen(db) = len;
+      SortLines(db_lines(db), db_lineslen(db));
+    }
+  }
+  else /* external, save leftpos and file pointer */
+  { db_filep(db) = fp;
+    left_pos(db) = leftp;
+  }
+
+  /* return */
+  debug1(DBS, DD, "] DbLoad returning %s", string(db));
   ifdebug(DPP, D, ProfileOff("DbLoad"));
   return db;
 } /* end DbLoad */
@@ -632,6 +666,54 @@ FULL_CHAR *str, FULL_CHAR *line)
 } /* end SearchFile */
 
 
+/*@::SearchLines()@***********************************************************/
+/*                                                                           */
+/*  static BOOLEAN SearchLines(LINE *lines, int left, int right, str, lnum)  */
+/*                                                                           */
+/*  Search the sorted array of LINE arrays lines[left..right] for a line     */
+/*  beginning with str, and return TRUE if found else FALSE.                 */
+/*                                                                           */
+/*  If TRUE is returned then the number of the line is in *lnum.             */
+/*                                                                           */
+/*****************************************************************************/
+
+static BOOLEAN SearchLines(LINE *lines, int left, int right, FULL_CHAR *str,
+  int *lnum)
+{ int l, r, mid;  FULL_CHAR buff[MAX_BUFF];
+  BOOLEAN res;
+  debug3(DBS, D, "SearchLines(lines, %d, %d, %s, lnum)", left, right, str);
+  if( right < left )
+  {
+    debug0(DBS, D, "SearchLines returning FALSE (empty lines)");
+    return FALSE;
+  }
+  l = left;
+  r = right - 1;
+  while( l <= r )
+  {
+    /* loop invt: first key >= str (if any) lies in the range lines[l..r+1] */
+    /* and left <= l <= right and r < right                                 */
+    mid = (l + r) / 2;
+    debug4(DBS, D, "  [l %d, r %d] examining lines[%d] = %s", l, r, mid,
+      lines[mid]);
+    if( StringLessEqual(str, (FULL_CHAR *) lines[mid]) )  r = mid - 1;
+    else l = mid + 1;
+  }
+  sscanf( (char *) lines[l], "%[^\t]", buff);
+  if( StringEqual(str, buff) )
+  {
+    res = TRUE;
+    *lnum = l;
+    debug1(DBS, D, "SearchLines returning TRUE (lnum %d)", *lnum);
+  }
+  else
+  { res = FALSE;
+    debug0(DBS, D, "SearchLines returning FALSE");
+  }
+  return res;
+} /* end SearchLines */
+
+
 /*@::DbRetrieve()@************************************************************/
 /*                                                                           */
 /*  BOOLEAN DbRetrieve(db, gall, sym, tag, seq, dfnum, dfpos, dlnum, cont)   */
@@ -643,27 +725,51 @@ FULL_CHAR *str, FULL_CHAR *line)
 /*****************************************************************************/
 
 BOOLEAN DbRetrieve(OBJECT db, BOOLEAN gall, OBJECT sym, FULL_CHAR *tag,
-FULL_CHAR *seq, FILE_NUM *dfnum, long *dfpos, int *dlnum, long *cont)
-{ int symnum;  FULL_CHAR line[MAX_BUFF], buff[MAX_BUFF];  OBJECT y;
+  FULL_CHAR *seq, FILE_NUM *dfnum, long *dfpos, int *dlnum, long *cont)
+{ int symnum, lnum;  FULL_CHAR line[MAX_BUFF], buff[MAX_BUFF];
   ifdebug(DPP, D, ProfileOn("DbRetrieve"));
-  debug4(DBS, D, "DbRetrieve(%s, %s%s&%s)", string(db), gall ? "0" : "",
+  debug4(DBS, DD, "DbRetrieve(%s, %s%s&%s)", string(db), gall ? "0" : "",
 	SymName(sym), tag);
-  if( !reading(db) || filep(db) == null )
-  { debug0(DBS, D, "DbRetrieve returning FALSE (empty or not reading)");
+
+  /* check OK to proceed */
+  if( !reading(db) || db_filep(db) == null )
+  { debug0(DBS, DD, "DbRetrieve returning FALSE (empty or not reading)");
     ifdebug(DPP, D, ProfileOff("DbRetrieve"));
     return FALSE;
   }
+
+  /* convert parameters into search key */
   SymToNum(db, sym, symnum, FALSE);
   sprintf( (char *) buff, "%s%d&%s", gall ? "0" : "", symnum, tag);
-  fseek(filep(db), 0L, SEEK_END);
-  if( !SearchFile(filep(db), (int) left_pos(db), (int) ftell(filep(db)) - 1,
-	buff, line) )
-  { debug0(DBS, D, "DbRetrieve returning FALSE (key not present)");
-    ifdebug(DPP, D, ProfileOff("DbRetrieve"));
-    return FALSE;
+
+  if( in_memory(db) )
+  {
+    /* search internal table, return if not found; set *cont to continuation */
+    if( !SearchLines(db_lines(db), 0, db_lineslen(db) - 1, buff, &lnum) )
+    { debug0(DBS, DD, "DbRetrieve returning FALSE (key not present)");
+      ifdebug(DPP, D, ProfileOff("DbRetrieve"));
+      return FALSE;
+    }
+    sscanf( (char *) db_lines(db)[lnum],
+      "%*[^\t]\t%[^\t]\t%*[^\t]\t%ld\t%d\t%[^\n]", seq, dfpos, dlnum, buff);
+    *cont = lnum+1;
   }
-  sscanf( (char *) line,
-    "%*[^\t]\t%[^\t]\t%*[^\t]\t%ld\t%d\t%[^\n]", seq, dfpos, dlnum, buff);
+  else
+  {
+    /* search for key in file, return if not found; set *cont to continuatn */
+    fseek(db_filep(db), 0L, SEEK_END);
+    if( !SearchFile(db_filep(db), (int) left_pos(db),
+	   (int) ftell(db_filep(db)) - 1, buff, line) )
+    { debug0(DBS, DD, "DbRetrieve returning FALSE (key not present)");
+      ifdebug(DPP, D, ProfileOff("DbRetrieve"));
+      return FALSE;
+    }
+    sscanf( (char *) line,
+      "%*[^\t]\t%[^\t]\t%*[^\t]\t%ld\t%d\t%[^\n]", seq, dfpos, dlnum, buff);
+    *cont = ftell(db_filep(db));
+  }
+
+  /* work out file name if . abbreviation used, and possibly define file */
   if( StringEqual(buff, AsciiToFull(".")) )
   { StringCopy(buff, string(db));
   }
@@ -673,9 +779,9 @@ FULL_CHAR *seq, FILE_NUM *dfnum, long *dfpos, int *dlnum, long *cont)
     *dfnum = DefineFile(buff, DATA_SUFFIX, &fpos(db),
       DATABASE_FILE, SOURCE_PATH);
   }
-  *cont = ftell(filep(db));
-  Child(y, Down(db));
-  debug3(DBS, D, "DbRetrieve returning TRUE (in %s at %ld, line %d)",
+
+  /* return */
+  debug3(DBS, DD, "DbRetrieve returning TRUE (in %s at %ld, line %d)",
     FileName(*dfnum), *dfpos, *dlnum);
   ifdebug(DPP, D, ProfileOff("DbRetrieve"));
   return TRUE;
@@ -684,7 +790,7 @@ FULL_CHAR *seq, FILE_NUM *dfnum, long *dfpos, int *dlnum, long *cont)
 
 /*@::DbRetrieveNext()@********************************************************/
 /*                                                                           */
-/*  BOOLEAN DbRetrieveNext(db, gall, sym, tag, seq, dfnum, dfpos, dlnum,cont)*/
+/*  BOOLEAN DbRetrieveNext(db, gall, sym, tag, seq, dfnum, dfpos,dlnum,cont) */
 /*                                                                           */
 /*  Retrieve the entry of database db pointed to by *cont.                   */
 /*  Set *gall, *sym, *tag, *seq, *dfnum, *dlnum, *dfpos to the value.        */
@@ -693,25 +799,49 @@ FULL_CHAR *seq, FILE_NUM *dfnum, long *dfpos, int *dlnum, long *cont)
 /*****************************************************************************/
 
 BOOLEAN DbRetrieveNext(OBJECT db, BOOLEAN *gall, OBJECT *sym, FULL_CHAR *tag,
-FULL_CHAR *seq, FILE_NUM *dfnum, long *dfpos, int *dlnum, long *cont)
-{ FULL_CHAR line[MAX_BUFF], fname[MAX_BUFF]; int symnum;
+  FULL_CHAR *seq, FILE_NUM *dfnum, long *dfpos, int *dlnum, long *cont)
+{ FULL_CHAR line[MAX_BUFF], *cline, fname[MAX_BUFF]; int symnum;
   ifdebug(DPP, D, ProfileOn("DbRetrieveNext"));
-  debug2(DBS, D, "DbRetrieveNext( %s, %ld )", string(db), *cont);
+  debug2(DBS, DD, "DbRetrieveNext( %s, %ld )", string(db), *cont);
   assert(reading(db), "DbRetrieveNext: not reading");
-  if( filep(db) == null )
-  { debug0(DBS, D, "DbRetrieveNext returning FALSE (empty database)");
+
+  /* check OK to proceed */
+  if( db_filep(db) == null )
+  { debug0(DBS, DD, "DbRetrieveNext returning FALSE (empty database)");
     ifdebug(DPP, D, ProfileOff("DbRetrieveNext"));
     return FALSE;
   }
-  fseek(filep(db), *cont == 0L ? (long) left_pos(db) : *cont, SEEK_SET);
-  if( StringFGets(line, MAX_BUFF, filep(db)) == NULL )
-  { debug0(DBS, D, "DbRetrieveNext returning FALSE (no successor)");
-    ifdebug(DPP, D, ProfileOff("DbRetrieveNext"));
-    return FALSE;
+
+  if( in_memory(db) )
+  {
+    /* get next entry from internal database */
+    if( *cont >= db_lineslen(db) )
+    { debug0(DBS, DD, "DbRetrieveNext returning FALSE (no successor)");
+      ifdebug(DPP, D, ProfileOff("DbRetrieveNext"));
+      return FALSE;
+    }
+    cline = (FULL_CHAR *) db_lines(db)[*cont];
+    *gall = (cline[0] == '0' ? 1 : 0);
+    sscanf((char *)&cline[*gall], "%d&%[^\t]\t%[^\t]\t%*[^\t]\t%ld\t%d\t%[^\n]",
+      &symnum, tag, seq, dfpos, dlnum, fname);
+    *cont = *cont + 1;
   }
-  *gall = (line[0] == '0' ? 1 : 0);
-  sscanf( (char *) &line[*gall], "%d&%[^\t]\t%[^\t]\t%*[^\t]\t%ld\t%d\t%[^\n]",
-    &symnum, tag, seq, dfpos, dlnum, fname);
+  else
+  {
+    /* use *cont to find position of next entry; advance *cont */
+    fseek(db_filep(db), *cont == 0L ? (long) left_pos(db) : *cont, SEEK_SET);
+    if( StringFGets(line, MAX_BUFF, db_filep(db)) == NULL )
+    { debug0(DBS, DD, "DbRetrieveNext returning FALSE (no successor)");
+      ifdebug(DPP, D, ProfileOff("DbRetrieveNext"));
+      return FALSE;
+    }
+    *gall = (line[0] == '0' ? 1 : 0);
+    sscanf((char *)&line[*gall], "%d&%[^\t]\t%[^\t]\t%*[^\t]\t%ld\t%d\t%[^\n]",
+      &symnum, tag, seq, dfpos, dlnum, fname);
+    *cont = ftell(db_filep(db));
+  }
+
+  /* work out file name if . abbreviation used, and possibly define file */
   if( StringEqual(fname, AsciiToFull(".")) )
   { StringCopy(fname, string(db));
   }
@@ -721,8 +851,10 @@ FULL_CHAR *seq, FILE_NUM *dfnum, long *dfpos, int *dlnum, long *cont)
     *dfnum = DefineFile(fname, DATA_SUFFIX, &fpos(db),
       DATABASE_FILE, SOURCE_PATH);
   }
-  NumToSym(db, symnum, *sym);  *cont = ftell(filep(db));
-  debug3(DBS, D, "DbRetrieveNext returning TRUE (in %s at %ld, line %d)",
+  NumToSym(db, symnum, *sym);
+
+  /* return */
+  debug3(DBS, DD, "DbRetrieveNext returning TRUE (in %s at %ld, line %d)",
     FileName(*dfnum), *dfpos, *dlnum);
   ifdebug(DPP, D, ProfileOff("DbRetrieveNext"));
   return TRUE;
