@@ -1,7 +1,7 @@
 /*@z36.c:Hyphenation: Declarations@*******************************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -10,7 +10,7 @@
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,14 +20,14 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z36.c                                                      */
 /*  MODULE:       Hyphenation                                                */
 /*  EXTERNS:      Hyphenate()                                                */
 /*                                                                           */
 /*****************************************************************************/
-#include "externs"
+#include "externs.h"
 #define	NODE_MULT	4		/* what to multiply node indexes by  */
 #define MAX_CHAR	256		/* max chars represented in one char */
 #define TRIE_MAGIC	5361534
@@ -319,6 +319,7 @@ static FULL_CHAR findrep(int i, TRIE T)
 } /* end findrep */
 
 
+#if 0
 /*****************************************************************************/
 /*                                                                           */
 /*  static FULL_CHAR *TrieRetrieve(key, T)                                   */
@@ -356,6 +357,7 @@ static FULL_CHAR *TrieRetrieve(FULL_CHAR *key, TRIE T)
     curr_node = NODE_MULT*next_node;  i++;
   }
 } /* end TrieRetrieve */
+#endif
 
 
 /*****************************************************************************/
@@ -742,7 +744,18 @@ static TRIE TrieRead(LANGUAGE_NUM lnum, BOOLEAN *success)
   debug0(DFS, DD, "  calling DefineFile from TrieRead (1)");
   packed_fnum = DefineFile(string(fname), HYPH_PACKED_SUFFIX,
     &fpos(fname), HYPH_PACKED_FILE, HYPH_PATH);
-  packed_fp = OpenFile(packed_fnum, FALSE, FALSE);
+  if( InitializeAll )
+  {
+    /* initializing so want to reconstruct packed files */
+    /* thanks to Ian Jackson <ian@chiark.greenend.org.uk> for this */
+    packed_fp = NULL;
+  }
+  else
+  {
+    /* not initializing so use existing packed files if possible */
+    packed_fp = OpenFile(packed_fnum, FALSE, FALSE);
+  }
+
   if( packed_fp == NULL )
   {
     /* no packed file, so define and open unpacked one instead */
@@ -1025,7 +1038,11 @@ OBJECT Hyphenate(OBJECT x)
   for( link = Down(x);  link != x;  link = NextDown(link) )
   { Child(y, link);
     if( !is_word(type(y)) || string(y)[0] == '\0' || !word_hyph(y) )
+    {
+      if( type(y) == GAP_OBJ && mode(gap(y)) == HYPH_MODE )
+	nobreak(gap(y)) = FALSE;
       continue;
+    }
     debug1(DHY, DD, "Hyphenate() examining %s", EchoObject(y));
 
     /* determine T, the trie to use */
@@ -1054,13 +1071,19 @@ OBJECT Hyphenate(OBJECT x)
 
     /* if a - ended the run, hyphenate there only */
     if( key[stop] == CH_HYPHEN )
-    { next_link = NextDown(link);
+    {
+      /* actually, don't hyphenate if the hyphen is last in the word [thanks Uwe] */
+      if( key[stop+1] == '\0' )
+	continue;
+
+      next_link = NextDown(link);
       z = MakeWord(WORD, &key[stop+1], &fpos(y));
       word_font(z) = word_font(y);
       word_colour(z) = word_colour(y);
       word_language(z) = word_language(y);
       word_hyph(z) = word_hyph(y);
       underline(z) = underline(y);
+      debug1(DHY, DD, "Hyphenate (hyph case) making fragment %s", string(z));
       FontWordSize(z);
       Link(NextDown(link), z);
       New(z, GAP_OBJ);
@@ -1071,7 +1094,8 @@ OBJECT Hyphenate(OBJECT x)
       Link(z, MakeWord(WORD, STR_GAP_ZERO_HYPH, &fpos(y)));
       key[stop + 1] = '\0';
       FontWordSize(y);
-      link = PrevDown(next_link);
+      /* *** link = PrevDown(next_link); */
+      link = NextDown(link);
       continue;
     }
 
