@@ -1,7 +1,7 @@
 /*@z39.c:String Handler:AsciiToFull(), StringEqual(), etc.@*******************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -10,7 +10,7 @@
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,11 +20,11 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z39.c                                                      */
 /*  MODULE:       String Handler                                             */
-/*  EXTERNS:      AsciiToFull(), StringEqual(), StringLessEqual(),           */
+/*  EXTERNS:      AsciiToFull(), StringEqual(),                              */
 /*                StringCat(), StringCopy(), StringLength(),                 */
 /*                StringFOpen(), StringFPuts(), StringFGets(),               */
 /*                StringRemove(), StringRename(), StringBeginsWith(),        */
@@ -32,14 +32,13 @@
 /*                StringQuotedWord()                                         */
 /*                                                                           */
 /*****************************************************************************/
-#include "externs"
+#include "externs.h"
 
 
 /*****************************************************************************/
 /*                                                                           */
 /*          AsciiToFull(str)          Returns ASCII string as FULL_CHARs.    */
 /*  BOOLEAN StringEqual(a, b)         TRUE if strings a and b are equal      */
-/*  BOOLEAN StringLessEqual(a, b)     TRUE if string a <= string b           */
 /*          StringCat(a, b)           Catenate string b onto end of a        */
 /*          StringCopy(a, b)          Overwrite string a with string b       */
 /*          StringLength(a)           Length of string a                     */
@@ -54,6 +53,25 @@
 /*****************************************************************************/
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  int strcollcmp(char *a, char *b)                                         */
+/*                                                                           */
+/*  Written by Valery Ushakov (uwe).                                         */
+/*                                                                           */
+/*  Like strcoll, but returns 0 only iff strcmp returns 0.                   */
+/*  This allow to test for equality using only strcmp. --uwe                 */
+/*                                                                           */
+/*****************************************************************************/
+
+int strcollcmp(char *a, char *b)
+{
+  int order = strcoll (a, b);
+  if( order == 0 ) /* then disambiguate with strcmp */
+    order = strcmp (a, b);
+  return order;
+}
+
 /*@::StringBeginsWith(), StringContains(), StringInt(), StringFiveInt()@******/
 /*                                                                           */
 /*  BOOLEAN StringBeginsWith(str, pattern)                                   */
