@@ -1,7 +1,7 @@
 /*@z41.c:Object Input-Output:AppendToFile, ReadFromFile@**********************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.10)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -10,7 +10,7 @@
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 1, or (at your option)      */
+/*  the Free Software Foundation; either Version 2, or (at your option)      */
 /*  any later version.                                                       */
 /*                                                                           */
 /*  This program is distributed in the hope that it will be useful,          */
@@ -20,14 +20,14 @@
 /*                                                                           */
 /*  You should have received a copy of the GNU General Public License        */
 /*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
+/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
 /*  FILE:         z41.c                                                      */
 /*  MODULE:       Object Input-Output                                        */
 /*  EXTERNS:      ReadFromFile(), AppendToFile(), CloseFiles()               */
 /*                                                                           */
 /*****************************************************************************/
-#include "externs"
+#include "externs.h"
 
 static FILE_NUM	last_write_fnum = NO_FILE;
 static FILE	*last_write_fp  = null;
@@ -146,6 +146,7 @@ static BOOLEAN need_lvis(OBJECT sym)	/* true if @LVis needed before sym */
 static void WriteClosure(OBJECT x, int *linecount, FILE_NUM fnum, OBJECT env)
 { OBJECT y, link, z, sym;
   BOOLEAN npar_written, name_printed;
+  debug2(DIO, D, "[ WriteClosure(%s %s)", Image(type(x)), EchoObject(x));
 
   sym = actual(x);
   /* *** if( use_invocation(sym) == x ) *** */
@@ -221,8 +222,11 @@ static void WriteClosure(OBJECT x, int *linecount, FILE_NUM fnum, OBJECT env)
 	  else
 	  { StringFPuts(STR_SPACE, last_write_fp);
 	  }
-	  if( filter(sym) != nilobj )
+	  /* old version: if( filter(sym) != nilobj ) */
+	  if( filter(sym) != nilobj && type(z) == FILTERED ) /* ??? */
 	  {
+	    debug1(DIO, D, "  filter(sym) != nilobj, type(z) == %s",
+	      Image(type(z)));
 	    assert( type(z) == FILTERED, "WriteClosure:  filter!" );
 	    WriteObject(z, NO_PREC, linecount, fnum);
 	  }
@@ -254,6 +258,7 @@ static void WriteClosure(OBJECT x, int *linecount, FILE_NUM fnum, OBJECT env)
       name_printed = TRUE;
     }
   }
+  debug0(DIO, D, "] WriteClosure returning");
 } /* end WriteClosure */
 
 
@@ -269,8 +274,9 @@ static void WriteClosure(OBJECT x, int *linecount, FILE_NUM fnum, OBJECT env)
 /*****************************************************************************/
 
 static void WriteObject(OBJECT x, int outer_prec, int *linecount, FILE_NUM fnum)
-{ OBJECT link, y, gap_obj, sym, env;  FULL_CHAR *name; int offset, lnum;
+{ OBJECT link, y, z, gap_obj, sym, env;  FULL_CHAR *name; int offset, lnum;
   int prec, i, last_prec;  BOOLEAN braces_needed;
+  debug2(DIO, D, "[ WriteObject(%s %s)", Image(type(x)), EchoObject(x));
   switch( type(x) )
   {
 
@@ -335,7 +341,7 @@ static void WriteObject(OBJECT x, int outer_prec, int *linecount, FILE_NUM fnum)
 				? JUXTA_PREC : ACAT_PREC;
 	    }
 	    else next_prec = prec;
-	    WriteObject(y, max(last_prec, next_prec), linecount, fnum);
+	    WriteObject(y, find_max(last_prec, next_prec), linecount, fnum);
 	  }
 	  else WriteObject(y, prec, linecount, fnum);
 	}
@@ -495,12 +501,20 @@ static void WriteObject(OBJECT x, int outer_prec, int *linecount, FILE_NUM fnum)
     case KERN_SHRINK:	name = KW_KERN_SHRINK;	goto SETC;
     case HCONTRACT:	name = KW_HCONTRACT;	goto SETC;
     case VCONTRACT:	name = KW_VCONTRACT;	goto SETC;
+    case HLIMITED:	name = KW_HLIMITED;	goto SETC;
+    case VLIMITED:	name = KW_VLIMITED;	goto SETC;
     case HEXPAND:	name = KW_HEXPAND;	goto SETC;
     case VEXPAND:	name = KW_VEXPAND;	goto SETC;
+    case START_HVSPAN:	name = KW_STARTHVSPAN;	goto SETC;
+    case START_HSPAN:	name = KW_STARTHSPAN;	goto SETC;
+    case START_VSPAN:	name = KW_STARTVSPAN;	goto SETC;
+    case HSPAN:		name = KW_HSPAN;	goto SETC;
+    case VSPAN:		name = KW_VSPAN;	goto SETC;
     case PADJUST:	name = KW_PADJUST;	goto SETC;
     case HADJUST:	name = KW_HADJUST;	goto SETC;
     case VADJUST:	name = KW_VADJUST;	goto SETC;
     case ROTATE:	name = KW_ROTATE;	goto SETC;
+    case BACKGROUND:	name = KW_BACKGROUND;	goto SETC;
     case CASE:		name = KW_CASE;		goto SETC;
     case YIELD:		name = KW_YIELD;	goto SETC;
     case BACKEND:	name = KW_BACKEND;	goto SETC;
@@ -514,9 +528,13 @@ static void WriteObject(OBJECT x, int outer_prec, int *linecount, FILE_NUM fnum)
     case COLOUR:	name = KW_COLOUR;	goto SETC;
     case LANGUAGE:	name = KW_LANGUAGE;	goto SETC;
     case CURR_LANG:	name = KW_CURR_LANG;	goto SETC;
+    case CURR_FAMILY:	name = KW_CURR_FAMILY;	goto SETC;
+    case CURR_FACE:	name = KW_CURR_FACE;	goto SETC;
     case COMMON:	name = KW_COMMON;	goto SETC;
     case RUMP:		name = KW_RUMP;		goto SETC;
+    case MELD:		name = KW_MELD;		goto SETC;
     case INSERT:	name = KW_INSERT;	goto SETC;
+    case ONE_OF:	name = KW_ONE_OF;	goto SETC;
     case NEXT:		name = KW_NEXT;		goto SETC;
     case PLUS:		name = KW_PLUS;		goto SETC;
     case MINUS:		name = KW_MINUS;	goto SETC;
@@ -524,6 +542,7 @@ static void WriteObject(OBJECT x, int outer_prec, int *linecount, FILE_NUM fnum)
     case TAGGED:	name = KW_TAGGED;	goto SETC;
     case INCGRAPHIC:	name = KW_INCGRAPHIC;	goto SETC;
     case SINCGRAPHIC:	name = KW_SINCGRAPHIC;	goto SETC;
+    case PLAIN_GRAPHIC:	name = KW_PLAINGRAPHIC;	goto SETC;
     case GRAPHIC:	name = KW_GRAPHIC;	goto SETC;
 
       /* print left parameter, if present */
@@ -553,6 +572,37 @@ static void WriteObject(OBJECT x, int outer_prec, int *linecount, FILE_NUM fnum)
       break;
 
 
+    case RAW_VERBATIM:
+    case VERBATIM:
+
+      StringFPuts(type(x) == VERBATIM ? KW_VERBATIM : KW_RAWVERBATIM, last_write_fp);
+      StringFPuts(STR_SPACE, last_write_fp);
+      StringFPuts(KW_BEGIN, last_write_fp);
+      StringFPuts(STR_NEWLINE, last_write_fp);
+      Child(y, Down(x));
+      if( type(y) == WORD )
+      {
+	StringFPuts(string(y), last_write_fp);
+        StringFPuts(STR_SPACE, last_write_fp);
+      }
+      else
+      {
+	assert( type(y) == VCAT, "WriteObject/VERBATIM!" );
+	for( link = Down(y);  link != y;  link = NextDown(link) )
+	{ Child(z, link);
+	  if( type(z) == GAP_OBJ )  continue;
+	  assert( type(z) == WORD, "WriteObject/VERBATIM/WORD!");
+	  StringFPuts(string(z), last_write_fp);
+	  StringFPuts(STR_NEWLINE, last_write_fp);
+	  *linecount += 1;
+	}
+      }
+      StringFPuts(KW_END, last_write_fp);
+      StringFPuts(STR_SPACE, last_write_fp);
+      StringFPuts(type(x) == VERBATIM ? KW_VERBATIM : KW_RAWVERBATIM, last_write_fp);
+      break;
+
+
     case FILTERED:
 
       FilterWrite(x, last_write_fp, linecount);
@@ -565,6 +615,7 @@ static void WriteObject(OBJECT x, int outer_prec, int *linecount, FILE_NUM fnum)
       break;
 
   } /* end switch */
+  debug0(DIO, D, "] WriteObject returning");
 } /* end WriteObject */
 
 
@@ -579,7 +630,7 @@ static void WriteObject(OBJECT x, int outer_prec, int *linecount, FILE_NUM fnum)
 
 void AppendToFile(OBJECT x, FILE_NUM fnum, int *pos, int *lnum)
 { FULL_CHAR buff[MAX_BUFF], *str;  int linecount;
-  debug2(DIO, D, "AppendToFile( %s, %s )", EchoObject(x), FileName(fnum));
+  debug2(DIO, D, "[ AppendToFile( %s, %s )", EchoObject(x), FileName(fnum));
 
   /* open file fnum for writing */
   if( last_write_fnum != fnum )
@@ -589,7 +640,8 @@ void AppendToFile(OBJECT x, FILE_NUM fnum, int *pos, int *lnum)
       Error(41, 3, "file name %s%s is too long",
 	FATAL, PosOfFile(fnum), str, NEW_DATA_SUFFIX);
     StringCopy(buff, str);  StringCat(buff, NEW_DATA_SUFFIX);
-    last_write_fp = StringFOpen(buff, APPEND_TEXT);
+    last_write_fp = StringFOpen(buff,
+      FileTestUpdated(fnum) ? APPEND_TEXT : WRITE_TEXT);
     if( last_write_fp == null )
       Error(41, 4, "cannot append to database file %s", FATAL, no_fpos, buff);
     last_write_fnum = fnum;
@@ -606,7 +658,7 @@ void AppendToFile(OBJECT x, FILE_NUM fnum, int *pos, int *lnum)
   StringFPuts(STR_NEWLINE, last_write_fp);
   StringFPuts(STR_NEWLINE, last_write_fp);
   FileSetUpdated(fnum, linecount + 2);
-  debug0(DIO, D, "AppendToFile returning.");
+  debug0(DIO, D, "] AppendToFile returning.");
 } /* end AppendToFile */
 
 
