@@ -1,7 +1,7 @@
 /*@z42.c:Colour Service:ColourChange, ColourCommand@**************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.12)                       */
-/*  COPYRIGHT (C) 1991, 1996 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.17)                       */
+/*  COPYRIGHT (C) 1991, 1999 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -195,8 +195,9 @@ void ColourChange(STYLE *style, OBJECT x)
 
   /* if argument is empty, fail and exit */
   if( StringEqual(string(x), STR_EMPTY) )
-  { Error(42, 4, "%s ignored (empty left parameter)", WARN, &fpos(x),
-      KW_COLOUR);
+  { if( BackEnd != PLAINTEXT )
+      Error(42, 4, "%s ignored (empty left parameter)", WARN, &fpos(x),
+        KW_COLOUR);
     debug0(DCO, D, "ColourChange returning (colour unchanged)");
     return;
   }
